'''
Use pre-accept hook to verify that both requests are made over the same TLS session
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import os

Test.Summary = '''
Verify that the client-side keep alive is honored for TLS and different versions of HTTP
'''

Test.SkipUnless(Condition.HasCurlFeature('http2'))

ts = Test.MakeATSProcess("ts", enable_tls=True)
server = Test.MakeOriginServer("server")
request_header = {"headers": "GET / HTTP/1.1\r\nHost: www.example.com\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
# desired response form the origin server
response_header = {"headers": "HTTP/1.1 200 OK\r\nConnection: close\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
server.addResponse("sessionlog.json", request_header, response_header)

ts.addSSLfile("ssl/server.pem")
ts.addSSLfile("ssl/server.key")

ts.Disk.records_config.update(
    {
        'proxy.config.ssl.server.cert.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.ssl.server.private_key.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.ssl.TLSv1_3.enabled': 0,
        'proxy.config.exec_thread.autoconfig.scale': 1.0,
        'proxy.config.log.max_secs_per_buffer': 1
    })

ts.Disk.ssl_multicert_config.AddLine('dest_ip=* ssl_cert_name=server.pem ssl_key_name=server.key')

ts.Disk.remap_config.AddLine(
    'map https://example.com:{0} http://127.0.0.1:{1}'.format(ts.Variables.ssl_port, server.Variables.Port))

ts.Disk.logging_yaml.AddLines(
    '''
logging:
  formats:
    - name: testformat
      format: '%<cqssl> %<cqtr>'
  logs:
    - mode: ascii
      format: testformat
      filename: squid
'''.split("\n"))

Test.PrepareTestPlugin(os.path.join(Test.Variables.AtsTestPluginsDir, 'ssl_secret_load_test.so'), ts)

tr = Test.AddTestRun("Test two HTTP/1.1 requests over one TLS connection")
tr.Processes.Default.StartBefore(server)
tr.Processes.Default.StartBefore(Test.Processes.ts)
tr.StillRunningAfter = ts
tr.StillRunningAfter = server
tr.Processes.Default.Command = 'curl -k -v --http1.1  -H \'host:example.com:{0}\' https://127.0.0.1:{0} https://127.0.0.1:{0}'.format(
    ts.Variables.ssl_port)
tr.Processes.Default.ReturnCode = 0

tr = Test.AddTestRun("Test two HTTP/1.1 requests over two TLS connections")
tr.StillRunningAfter = ts
tr.StillRunningAfter = server
tr.Processes.Default.Command = 'curl -k -v --http1.1  -H \'host:example.com:{0}\' https://127.0.0.1:{0}; curl -k -v --http1.1 -H \'host:example.com:{0}\'  https://127.0.0.1:{0}'.format(
    ts.Variables.ssl_port)
tr.Processes.Default.ReturnCode = 0

tr = Test.AddTestRun("Test two HTTP/2 requests over one TLS connection")
tr.StillRunningAfter = ts
tr.StillRunningAfter = server
tr.Processes.Default.Command = 'curl -k -v --http2  -H \'host:example.com:{0}\' https://127.0.0.1:{0} https://127.0.0.1:{0}'.format(
    ts.Variables.ssl_port)
tr.Processes.Default.ReturnCode = 0

tr = Test.AddTestRun("Test two HTTP/2 requests over two TLS connections")
tr.StillRunningAfter = ts
tr.StillRunningAfter = server
tr.Processes.Default.Command = 'curl -k -v --http2  -H \'host:example.com:{0}\' https://127.0.0.1:{0}; curl -k -v --http1.1 -H \'host:example.com:{0}\'  https://127.0.0.1:{0}'.format(
    ts.Variables.ssl_port)
tr.Processes.Default.ReturnCode = 0

# Just a check to flush out the traffic log until we have a clean shutdown for traffic_server
tr = Test.AddTestRun("Wait for the access log to write out")
tr.DelayStart = 5
tr.StillRunningAfter = ts
tr.StillRunningAfter = server
tr.Processes.Default.Command = 'ls'
tr.Processes.Default.ReturnCode = 0

ts.Disk.squid_log.Content = "gold/accesslog.gold"



#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import os

Test.Summary = '''
Test spawning a thread in a transaction hook continuation, and getting a result from it, without blocking event task.
'''

plugin_name = "polite_hook_wait"

server = Test.MakeOriginServer("server")

request_header = {"headers": "GET / HTTP/1.1\r\nHost: doesnotmatter\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
response_header = {"headers": "HTTP/1.1 200 OK\r\nConnection: close\r\n\r\n", "timestamp": "1469733493.993", "body": "112233"}
server.addResponse("sessionlog.json", request_header, response_header)

# Disable the cache to make sure each request is forwarded to the origin
# server.
ts = Test.MakeATSProcess("ts", enable_cache=False, block_for_debug=False)

ts.Disk.records_config.update(
    {
        'proxy.config.proxy_name': 'Poxy_Proxy',  # This will be the server name.
        'proxy.config.url_remap.remap_required': 1,
        'proxy.config.diags.debug.enabled': 3,
        'proxy.config.diags.debug.tags': f'{plugin_name}',
    })

rp = os.path.join(Test.Variables.AtsBuildGoldTestsDir, 'pluginTest', 'polite_hook_wait', '.libs', f'{plugin_name}.so')
ts.Setup.Copy(rp, ts.Env['PROXY_CONFIG_PLUGIN_PLUGIN_DIR'])

ts.Disk.plugin_config.AddLine(f"{plugin_name}.so")

ts.Disk.remap_config.AddLine("map http://myhost.test http://127.0.0.1:{0}".format(server.Variables.Port))

tr = Test.AddTestRun()
tr.Processes.Default.StartBefore(server)
tr.Processes.Default.StartBefore(ts)
tr.Processes.Default.Command = (
    'curl --verbose --ipv4 --header "Host:myhost.test" http://localhost:{}/ 2>curl.txt'.format(ts.Variables.port))
tr.Processes.Default.ReturnCode = 0

tr = Test.AddTestRun()
tr.Processes.Default.Command = (
    'curl --verbose --ipv4 --header "Host:myhost.test" http://localhost:{}/ 2>curl.txt'.format(ts.Variables.port))
tr.Processes.Default.ReturnCode = 0

tr = Test.AddTestRun()
tr.Processes.Default.Command = "grep -F HTTP/ curl.txt"
tr.Processes.Default.Streams.stdout = "curl.gold"
tr.Processes.Default.ReturnCode = 0



'''
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import os

Test.Summary = '''
Test log filter.
'''

ts = Test.MakeATSProcess("ts", enable_cache=False)
replay_file = "log-filter.replays.yaml"
server = Test.MakeVerifierServerProcess("server", replay_file)
nameserver = Test.MakeDNServer("dns", default='127.0.0.1')

ts.Disk.records_config.update(
    {
        'proxy.config.diags.debug.enabled': 2,
        'proxy.config.diags.debug.tags': 'http',
        'proxy.config.diags.debug.client_ip': '127.0.0.1',
        'proxy.config.dns.nameservers': f"127.0.0.1:{nameserver.Variables.Port}",
    })
ts.Disk.remap_config.AddLine('map / http://localhost:{}/'.format(server.Variables.http_port))

# Verify that the various aspects of the expected debug output for the
# transaction are logged.
ts.Disk.traffic_out.Content = Testers.ContainsExpression(
    r"\+ Incoming Request \+", "Make sure the client request information is present.")
ts.Disk.traffic_out.Content += Testers.ContainsExpression(
    r"\+ Proxy's Request after hooks \+", "Make sure the proxy request information is present.")
ts.Disk.traffic_out.Content += Testers.ContainsExpression(
    r"\+ Incoming O.S. Response \+", "Make sure the server's response information is present.")
ts.Disk.traffic_out.Content += Testers.ContainsExpression(
    r"\+ Proxy's Response 2 \+", "Make sure the proxy response information is present.")

tr = Test.AddTestRun()
tr.Processes.Default.StartBefore(server)
tr.Processes.Default.StartBefore(ts)
tr.Processes.Default.StartBefore(nameserver)
tr.AddVerifierClientProcess("client-1", replay_file, http_ports=[ts.Variables.port], other_args="--keys test-1")



'''
Test SERVER_VERIFY_HOOK
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import os

Test.Summary = '''
Test different combinations of TLS handshake hooks to ensure they are applied consistently.
'''

Test.SkipUnless(Condition.HasOpenSSLVersion("1.1.1"))

ts = Test.MakeATSProcess("ts", enable_tls=True)
server = Test.MakeOriginServer("server", ssl=True)
request_header = {"headers": "GET / HTTP/1.1\r\nHost: www.example.com\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
# desired response form the origin server
response_header = {"headers": "HTTP/1.1 200 OK\r\nConnection: close\r\n\r\n", "timestamp": "1469733493.993", "body": ""}
server.addResponse("sessionlog.json", request_header, response_header)

ts.addSSLfile("ssl/server.pem")
ts.addSSLfile("ssl/server.key")
ts.addSSLfile("ssl/signer.pem")

ts.Disk.records_config.update(
    {
        # Test looks for debug output from the plugin
        'proxy.config.diags.debug.enabled': 1,
        'proxy.config.diags.debug.tags': 'ssl_client_verify_test',
        'proxy.config.ssl.server.cert.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.ssl.server.private_key.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.exec_thread.autoconfig.scale': 1.0,
        'proxy.config.ssl.CA.cert.filename': '{0}/signer.pem'.format(ts.Variables.SSLDir),
        'proxy.config.url_remap.pristine_host_hdr': 1
    })

ts.Disk.ssl_multicert_config.AddLine('dest_ip=* ssl_cert_name=server.pem ssl_key_name=server.key')

ts.Disk.remap_config.AddLine(
    'map https://foo.com:{1}/ https://127.0.0.1:{0}'.format(server.Variables.SSL_Port, ts.Variables.ssl_port))
ts.Disk.remap_config.AddLine(
    'map https://bar.com:{1}/ https://127.0.0.1:{0}'.format(server.Variables.SSL_Port, ts.Variables.ssl_port))
ts.Disk.remap_config.AddLine(
    'map https://random.com:{1}/ https://127.0.0.1:{0}'.format(server.Variables.SSL_Port, ts.Variables.ssl_port))

ts.Disk.sni_yaml.AddLines([
    'sni:',
    '- fqdn: bar.com',
    '  verify_client: STRICT',
    '- fqdn: foo.com',
    '  verify_client: STRICT',
])

Test.PrepareTestPlugin(os.path.join(Test.Variables.AtsTestPluginsDir, 'ssl_client_verify_test.so'), ts, '-count=2 -good=foo.com')

tr = Test.AddTestRun("request good name")
tr.Setup.Copy("ssl/signed-foo.pem")
tr.Setup.Copy("ssl/signed-foo.key")
tr.Setup.Copy("ssl/signed-bar.pem")
tr.Setup.Copy("ssl/signed-bar.key")
tr.Processes.Default.StartBefore(server)
tr.Processes.Default.StartBefore(Test.Processes.ts)
tr.StillRunningAfter = ts
tr.StillRunningAfter = server
tr.Processes.Default.Command = "curl --tls-max 1.2 -k --cert ./signed-foo.pem --key ./signed-foo.key --resolve 'foo.com:{0}:127.0.0.1' https://foo.com:{0}/case1".format(
    ts.Variables.ssl_port)
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.all = Testers.ExcludesExpression("Could Not Connect", "Curl attempt should have succeeded")

tr2 = Test.AddTestRun("request bad name")
tr2.StillRunningAfter = ts
tr2.StillRunningAfter = server
tr2.Processes.Default.Command = "curl --tls-max 1.2 -k --cert ./signed-bar.pem --key ./signed-bar.key --resolve 'foo.com:{0}:127.0.0.1' https://foo.com:{0}/case1".format(
    ts.Variables.ssl_port)
tr2.Processes.Default.ReturnCode = 35
tr2.Processes.Default.Streams.all = Testers.ContainsExpression("error", "Curl attempt should have failed")

tr3 = Test.AddTestRun("request badly signed cert")
tr3.Setup.Copy("ssl/server.pem")
tr3.Setup.Copy("ssl/server.key")
tr3.StillRunningAfter = ts
tr3.StillRunningAfter = server
tr3.Processes.Default.Command = "curl --tls-max 1.2 -k --cert ./server.pem --key ./server.key --resolve 'foo.com:{0}:127.0.0.1' https://foo.com:{0}/case1".format(
    ts.Variables.ssl_port)
tr3.Processes.Default.ReturnCode = 35
tr3.Processes.Default.Streams.all = Testers.ContainsExpression("error", "Curl attempt should have failed")

ts.Disk.traffic_out.Content += Testers.ContainsExpression(
    r"Client verify callback 0 [\da-fx]+? - event is good good HS", "verify callback happens 2 times")
ts.Disk.traffic_out.Content += Testers.ContainsExpression(
    r"Client verify callback 1 [\da-fx]+? - event is good good HS", "verify callback happens 2 times")
ts.Disk.traffic_out.Content += Testers.ContainsExpression(
    r"Client verify callback 0 [\da-fx]+? - event is good error HS", "verify callback happens 2 times")
ts.Disk.traffic_out.Content += Testers.ContainsExpression(
    r"Client verify callback 1 [\da-fx]+? - event is good error HS", "verify callback happens 2 times")



'''
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import os
import re

Test.Summary = 'Test the STEK Share plugin'
Test.testName = "stek_share"

Test.SkipUnless(Condition.PluginExists('stek_share.so'))

server = Test.MakeOriginServer('server')

ts1 = Test.MakeATSProcess("ts1", enable_tls=True)
ts2 = Test.MakeATSProcess("ts2", enable_tls=True)
ts3 = Test.MakeATSProcess("ts3", enable_tls=True)
ts4 = Test.MakeATSProcess("ts4", enable_tls=True)
ts5 = Test.MakeATSProcess("ts5", enable_tls=True)

Test.Setup.Copy('ssl/self_signed.crt')
Test.Setup.Copy('ssl/self_signed.key')
Test.Setup.Copy('server_list.yaml')

cert_path = os.path.join(Test.RunDirectory, 'self_signed.crt')
key_path = os.path.join(Test.RunDirectory, 'self_signed.key')
server_list_path = os.path.join(Test.RunDirectory, 'server_list.yaml')

request_header1 = {'headers': 'GET / HTTP/1.1\r\nHost: www.example.com\r\n\r\n', 'timestamp': '1469733493.993', 'body': ''}
response_header1 = {'headers': 'HTTP/1.1 200 OK\r\nConnection: close\r\n\r\n', 'timestamp': '1469733493.993', 'body': 'curl test'}
server.addResponse('sessionlog.json', request_header1, response_header1)

stek_share_conf_path_1 = os.path.join(ts1.Variables.CONFIGDIR, 'stek_share_conf.yaml')
stek_share_conf_path_2 = os.path.join(ts2.Variables.CONFIGDIR, 'stek_share_conf.yaml')
stek_share_conf_path_3 = os.path.join(ts3.Variables.CONFIGDIR, 'stek_share_conf.yaml')
stek_share_conf_path_4 = os.path.join(ts4.Variables.CONFIGDIR, 'stek_share_conf.yaml')
stek_share_conf_path_5 = os.path.join(ts5.Variables.CONFIGDIR, 'stek_share_conf.yaml')

ts1.Disk.File(stek_share_conf_path_1, id="stek_share_conf_1", typename="ats:config")
ts2.Disk.File(stek_share_conf_path_2, id="stek_share_conf_2", typename="ats:config")
ts3.Disk.File(stek_share_conf_path_3, id="stek_share_conf_3", typename="ats:config")
ts4.Disk.File(stek_share_conf_path_4, id="stek_share_conf_4", typename="ats:config")
ts5.Disk.File(stek_share_conf_path_5, id="stek_share_conf_5", typename="ats:config")

ts1.Disk.stek_share_conf_1.AddLines(
    [
        'server_id: 1',
        'address: 127.0.0.1',
        'port: 10001',
        'asio_thread_pool_size: 4',
        'heart_beat_interval: 100',
        'election_timeout_lower_bound: 200',
        'election_timeout_upper_bound: 400',
        'reserved_log_items: 5',
        'snapshot_distance: 5',
        'client_req_timeout: 3000',  # this is in milliseconds
        'key_update_interval: 3600',  # this is in seconds
        'server_list_file: {0}'.format(server_list_path),
        'root_cert_file: {0}'.format(cert_path),
        'server_cert_file: {0}'.format(cert_path),
        'server_key_file: {0}'.format(key_path),
        'cert_verify_str: /C=US/ST=IL/O=Yahoo/OU=Edge/CN=stek-share',
    ])

ts2.Disk.stek_share_conf_2.AddLines(
    [
        'server_id: 2',
        'address: 127.0.0.1',
        'port: 10002',
        'asio_thread_pool_size: 4',
        'heart_beat_interval: 100',
        'election_timeout_lower_bound: 200',
        'election_timeout_upper_bound: 400',
        'reserved_log_items: 5',
        'snapshot_distance: 5',
        'client_req_timeout: 3000',  # this is in milliseconds
        'key_update_interval: 3600',  # this is in seconds
        'server_list_file: {0}'.format(server_list_path),
        'root_cert_file: {0}'.format(cert_path),
        'server_cert_file: {0}'.format(cert_path),
        'server_key_file: {0}'.format(key_path),
        'cert_verify_str: /C=US/ST=IL/O=Yahoo/OU=Edge/CN=stek-share',
    ])

ts3.Disk.stek_share_conf_3.AddLines(
    [
        'server_id: 3',
        'address: 127.0.0.1',
        'port: 10003',
        'asio_thread_pool_size: 4',
        'heart_beat_interval: 100',
        'election_timeout_lower_bound: 200',
        'election_timeout_upper_bound: 400',
        'reserved_log_items: 5',
        'snapshot_distance: 5',
        'client_req_timeout: 3000',  # this is in milliseconds
        'key_update_interval: 3600',  # this is in seconds
        'server_list_file: {0}'.format(server_list_path),
        'root_cert_file: {0}'.format(cert_path),
        'server_cert_file: {0}'.format(cert_path),
        'server_key_file: {0}'.format(key_path),
        'cert_verify_str: /C=US/ST=IL/O=Yahoo/OU=Edge/CN=stek-share',
    ])

ts4.Disk.stek_share_conf_4.AddLines(
    [
        'server_id: 4',
        'address: 127.0.0.1',
        'port: 10004',
        'asio_thread_pool_size: 4',
        'heart_beat_interval: 100',
        'election_timeout_lower_bound: 200',
        'election_timeout_upper_bound: 400',
        'reserved_log_items: 5',
        'snapshot_distance: 5',
        'client_req_timeout: 3000',  # this is in milliseconds
        'key_update_interval: 3600',  # this is in seconds
        'server_list_file: {0}'.format(server_list_path),
        'root_cert_file: {0}'.format(cert_path),
        'server_cert_file: {0}'.format(cert_path),
        'server_key_file: {0}'.format(key_path),
        'cert_verify_str: /C=US/ST=IL/O=Yahoo/OU=Edge/CN=stek-share',
    ])

ts5.Disk.stek_share_conf_5.AddLines(
    [
        'server_id: 5',
        'address: 127.0.0.1',
        'port: 10005',
        'asio_thread_pool_size: 4',
        'heart_beat_interval: 100',
        'election_timeout_lower_bound: 200',
        'election_timeout_upper_bound: 400',
        'reserved_log_items: 5',
        'snapshot_distance: 5',
        'client_req_timeout: 3000',  # this is in milliseconds
        'key_update_interval: 3600',  # this is in seconds
        'server_list_file: {0}'.format(server_list_path),
        'root_cert_file: {0}'.format(cert_path),
        'server_cert_file: {0}'.format(cert_path),
        'server_key_file: {0}'.format(key_path),
        'cert_verify_str: /C=US/ST=IL/O=Yahoo/OU=Edge/CN=stek-share',
    ])

ts1.Disk.records_config.update(
    {
        'proxy.config.diags.debug.enabled': 1,
        'proxy.config.diags.debug.tags': 'stek_share',
        'proxy.config.exec_thread.autoconfig.enabled': 0,
        'proxy.config.exec_thread.limit': 4,
        'proxy.config.ssl.server.cert.path': '{0}'.format(Test.RunDirectory),
        'proxy.config.ssl.server.private_key.path': '{0}'.format(Test.RunDirectory),
        'proxy.config.ssl.session_cache.value': 2,
        'proxy.config.ssl.session_cache.size': 1024,
        'proxy.config.ssl.session_cache.timeout': 7200,
        'proxy.config.ssl.session_cache.num_buckets': 16,
        'proxy.config.ssl.server.session_ticket.enable': 1,
        'proxy.config.ssl.server.cipher_suite':
            'ECDHE-ECDSA-AES256-GCM-SHA384:ECDHE-RSA-AES256-GCM-SHA384:ECDHE-ECDSA-AES128-GCM-SHA256:ECDHE-RSA-AES128-GCM-SHA256:DHE-RSA-AES256-GCM-SHA384:DHE-DSS-AES256-GCM-SHA384:DHE-RSA-AES128-GCM-SHA256:DHE-DSS-AES128-GCM-SHA256:ECDHE-ECDSA-AES256-SHA384:ECDHE-RSA-AES256-SHA384:ECDHE-ECDSA-AES256-SHA:ECDHE-RSA-AES256-SHA:ECDHE-ECDSA-AES128-SHA256:ECDHE-RSA-AES128-SHA256:ECDHE-ECDSA-AES128-SHA:ECDHE-RSA-AES128-SHA:DHE-RSA-AES256-SHA256:DHE-DSS-AES256-SHA256:DHE-RSA-AES128-SHA256:DHE-DSS-AES128-SHA256:DHE-RSA-AES256-SHA:DHE-DSS-AES256-SHA:DHE-RSA-AES128-SHA:DHE-DSS-AES128-SHA:!aNULL:!eNULL:!EXPORT:!DES:!RC4:!MD5:!PSK:!aECDH:!EDH-DSS-DES-CBC3-SHA:!EDH-RSA-DES-CBC3-SHA:!KRB5-DES-CBC3-SHA'
    })
ts1.Disk.plugin_config.AddLine('stek_share.so {0}'.format(stek_share_conf_path_1))
ts1.Disk.ssl_multicert_config.AddLine('dest_ip=* ssl_cert_name=self_signed.crt ssl_key_name=self_signed.key')
ts1.Disk.remap_config.AddLine('map / http://127.0.0.1:{0}'.format(server.Variables.Port))

ts2.Disk.records_config.update(
    {
        'proxy.config.diags.debug.enabled': 1,
        'proxy.config.diags.debug.tags': 'stek_share',
        'proxy.config.exec_thread.autoconfig.enabled': 0,
        'proxy.config.exec_thread.limit': 4,
        'proxy.config.ssl.server.cert.path': '{0}'.format(Test.RunDirectory),
        'proxy.config.ssl.server.private_key.path': '{0}'.format(Test.RunDirectory),
        'proxy.config.ssl.session_cache.value': 2,
        'proxy.config.ssl.session_cache.size': 1024,
        'proxy.config.ssl.session_cache.timeout': 7200,
        'proxy.config.ssl.session_cache.num_buckets': 16,
        'proxy.config.ssl.server.session_ticket.enable': 1,
        'proxy.config.ssl.server.cipher_suite':
            'ECDHE-ECDSA-AES256-GCM-SHA384:ECDHE-RSA-AES256-GCM-SHA384:ECDHE-ECDSA-AES128-GCM-SHA256:ECDHE-RSA-AES128-GCM-SHA256:DHE-RSA-AES256-GCM-SHA384:DHE-DSS-AES256-GCM-SHA384:DHE-RSA-AES128-GCM-SHA256:DHE-DSS-AES128-GCM-SHA256:ECDHE-ECDSA-AES256-SHA384:ECDHE-RSA-AES256-SHA384:ECDHE-ECDSA-AES256-SHA:ECDHE-RSA-AES256-SHA:ECDHE-ECDSA-AES128-SHA256:ECDHE-RSA-AES128-SHA256:ECDHE-ECDSA-AES128-SHA:ECDHE-RSA-AES128-SHA:DHE-RSA-AES256-SHA256:DHE-DSS-AES256-SHA256:DHE-RSA-AES128-SHA256:DHE-DSS-AES128-SHA256:DHE-RSA-AES256-SHA:DHE-DSS-AES256-SHA:DHE-RSA-AES128-SHA:DHE-DSS-AES128-SHA:!aNULL:!eNULL:!EXPORT:!DES:!RC4:!MD5:!PSK:!aECDH:!EDH-DSS-DES-CBC3-SHA:!EDH-RSA-DES-CBC3-SHA:!KRB5-DES-CBC3-SHA'
    })
ts2.Disk.plugin_config.AddLine('stek_share.so {0}'.format(stek_share_conf_path_2))
ts2.Disk.ssl_multicert_config.AddLine('dest_ip=* ssl_cert_name=self_signed.crt ssl_key_name=self_signed.key')
ts2.Disk.remap_config.AddLine('map / http://127.0.0.1:{0}'.format(server.Variables.Port))

ts3.Disk.records_config.update(
    {
        'proxy.config.diags.debug.enabled': 1,
        'proxy.config.diags.debug.tags': 'stek_share',
        'proxy.config.exec_thread.autoconfig.enabled': 0,
        'proxy.config.exec_thread.limit': 4,
        'proxy.config.ssl.server.cert.path': '{0}'.format(Test.RunDirectory),
        'proxy.config.ssl.server.private_key.path': '{0}'.format(Test.RunDirectory),
        'proxy.config.ssl.session_cache.value': 2,
        'proxy.config.ssl.session_cache.size': 1024,
        'proxy.config.ssl.session_cache.timeout': 7200,
        'proxy.config.ssl.session_cache.num_buckets': 16,
        'proxy.config.ssl.server.session_ticket.enable': 1,
        'proxy.config.ssl.server.cipher_suite':
            'ECDHE-ECDSA-AES256-GCM-SHA384:ECDHE-RSA-AES256-GCM-SHA384:ECDHE-ECDSA-AES128-GCM-SHA256:ECDHE-RSA-AES128-GCM-SHA256:DHE-RSA-AES256-GCM-SHA384:DHE-DSS-AES256-GCM-SHA384:DHE-RSA-AES128-GCM-SHA256:DHE-DSS-AES128-GCM-SHA256:ECDHE-ECDSA-AES256-SHA384:ECDHE-RSA-AES256-SHA384:ECDHE-ECDSA-AES256-SHA:ECDHE-RSA-AES256-SHA:ECDHE-ECDSA-AES128-SHA256:ECDHE-RSA-AES128-SHA256:ECDHE-ECDSA-AES128-SHA:ECDHE-RSA-AES128-SHA:DHE-RSA-AES256-SHA256:DHE-DSS-AES256-SHA256:DHE-RSA-AES128-SHA256:DHE-DSS-AES128-SHA256:DHE-RSA-AES256-SHA:DHE-DSS-AES256-SHA:DHE-RSA-AES128-SHA:DHE-DSS-AES128-SHA:!aNULL:!eNULL:!EXPORT:!DES:!RC4:!MD5:!PSK:!aECDH:!EDH-DSS-DES-CBC3-SHA:!EDH-RSA-DES-CBC3-SHA:!KRB5-DES-CBC3-SHA'
    })
ts3.Disk.plugin_config.AddLine('stek_share.so {0}'.format(stek_share_conf_path_3))
ts3.Disk.ssl_multicert_config.AddLine('dest_ip=* ssl_cert_name=self_signed.crt ssl_key_name=self_signed.key')
ts3.Disk.remap_config.AddLine('map / http://127.0.0.1:{0}'.format(server.Variables.Port))

ts4.Disk.records_config.update(
    {
        'proxy.config.diags.debug.enabled': 1,
        'proxy.config.diags.debug.tags': 'stek_share',
        'proxy.config.exec_thread.autoconfig.enabled': 0,
        'proxy.config.exec_thread.limit': 4,
        'proxy.config.ssl.server.cert.path': '{0}'.format(Test.RunDirectory),
        'proxy.config.ssl.server.private_key.path': '{0}'.format(Test.RunDirectory),
        'proxy.config.ssl.session_cache.value': 2,
        'proxy.config.ssl.session_cache.size': 1024,
        'proxy.config.ssl.session_cache.timeout': 7200,
        'proxy.config.ssl.session_cache.num_buckets': 16,
        'proxy.config.ssl.server.session_ticket.enable': 1,
        'proxy.config.ssl.server.cipher_suite':
            'ECDHE-ECDSA-AES256-GCM-SHA384:ECDHE-RSA-AES256-GCM-SHA384:ECDHE-ECDSA-AES128-GCM-SHA256:ECDHE-RSA-AES128-GCM-SHA256:DHE-RSA-AES256-GCM-SHA384:DHE-DSS-AES256-GCM-SHA384:DHE-RSA-AES128-GCM-SHA256:DHE-DSS-AES128-GCM-SHA256:ECDHE-ECDSA-AES256-SHA384:ECDHE-RSA-AES256-SHA384:ECDHE-ECDSA-AES256-SHA:ECDHE-RSA-AES256-SHA:ECDHE-ECDSA-AES128-SHA256:ECDHE-RSA-AES128-SHA256:ECDHE-ECDSA-AES128-SHA:ECDHE-RSA-AES128-SHA:DHE-RSA-AES256-SHA256:DHE-DSS-AES256-SHA256:DHE-RSA-AES128-SHA256:DHE-DSS-AES128-SHA256:DHE-RSA-AES256-SHA:DHE-DSS-AES256-SHA:DHE-RSA-AES128-SHA:DHE-DSS-AES128-SHA:!aNULL:!eNULL:!EXPORT:!DES:!RC4:!MD5:!PSK:!aECDH:!EDH-DSS-DES-CBC3-SHA:!EDH-RSA-DES-CBC3-SHA:!KRB5-DES-CBC3-SHA'
    })
ts4.Disk.plugin_config.AddLine('stek_share.so {0}'.format(stek_share_conf_path_4))
ts4.Disk.ssl_multicert_config.AddLine('dest_ip=* ssl_cert_name=self_signed.crt ssl_key_name=self_signed.key')
ts4.Disk.remap_config.AddLine('map / http://127.0.0.1:{0}'.format(server.Variables.Port))

ts5.Disk.records_config.update(
    {
        'proxy.config.diags.debug.enabled': 1,
        'proxy.config.diags.debug.tags': 'stek_share',
        'proxy.config.exec_thread.autoconfig.enabled': 0,
        'proxy.config.exec_thread.limit': 4,
        'proxy.config.ssl.server.cert.path': '{0}'.format(Test.RunDirectory),
        'proxy.config.ssl.server.private_key.path': '{0}'.format(Test.RunDirectory),
        'proxy.config.ssl.session_cache.value': 2,
        'proxy.config.ssl.session_cache.size': 1024,
        'proxy.config.ssl.session_cache.timeout': 7200,
        'proxy.config.ssl.session_cache.num_buckets': 16,
        'proxy.config.ssl.server.session_ticket.enable': 1,
        'proxy.config.ssl.server.cipher_suite':
            'ECDHE-ECDSA-AES256-GCM-SHA384:ECDHE-RSA-AES256-GCM-SHA384:ECDHE-ECDSA-AES128-GCM-SHA256:ECDHE-RSA-AES128-GCM-SHA256:DHE-RSA-AES256-GCM-SHA384:DHE-DSS-AES256-GCM-SHA384:DHE-RSA-AES128-GCM-SHA256:DHE-DSS-AES128-GCM-SHA256:ECDHE-ECDSA-AES256-SHA384:ECDHE-RSA-AES256-SHA384:ECDHE-ECDSA-AES256-SHA:ECDHE-RSA-AES256-SHA:ECDHE-ECDSA-AES128-SHA256:ECDHE-RSA-AES128-SHA256:ECDHE-ECDSA-AES128-SHA:ECDHE-RSA-AES128-SHA:DHE-RSA-AES256-SHA256:DHE-DSS-AES256-SHA256:DHE-RSA-AES128-SHA256:DHE-DSS-AES128-SHA256:DHE-RSA-AES256-SHA:DHE-DSS-AES256-SHA:DHE-RSA-AES128-SHA:DHE-DSS-AES128-SHA:!aNULL:!eNULL:!EXPORT:!DES:!RC4:!MD5:!PSK:!aECDH:!EDH-DSS-DES-CBC3-SHA:!EDH-RSA-DES-CBC3-SHA:!KRB5-DES-CBC3-SHA'
    })
ts5.Disk.plugin_config.AddLine('stek_share.so {0}'.format(stek_share_conf_path_5))
ts5.Disk.ssl_multicert_config.AddLine('dest_ip=* ssl_cert_name=self_signed.crt ssl_key_name=self_signed.key')
ts5.Disk.remap_config.AddLine('map / http://127.0.0.1:{0}'.format(server.Variables.Port))


def check_session(ev, test):
    retval = False
    f = open(test.GetContent(ev), 'r')
    err = "Session ids match"
    if not f:
        err = "Failed to open {0}".format(openssl_output)
        return (retval, "Check that session ids match", err)

    content = f.read()
    match = re.findall('Session-ID: ([0-9A-F]+)', content)

    if match:
        if all(i == j for i, j in zip(match, match[1:])):
            err = "{0} reused successfully {1} times".format(match[0], len(match) - 1)
            retval = True
        else:
            err = "Session is not being reused as expected"
    else:
        err = "Didn't find session id"
    return (retval, "Check that session ids match", err)


tr1 = Test.AddTestRun('Basic Curl test, and give it enough time for all ATS to start up and sync STEK')
tr1.Processes.Default.Command = 'sleep 10 && curl https://127.0.0.1:{0} -k'.format(ts1.Variables.ssl_port)
tr1.Processes.Default.ReturnCode = 0
tr1.Processes.Default.StartBefore(server)
tr1.Processes.Default.StartBefore(ts1)
tr1.Processes.Default.StartBefore(ts2)
tr1.Processes.Default.StartBefore(ts3)
tr1.Processes.Default.StartBefore(ts4)
tr1.Processes.Default.StartBefore(ts5)
tr1.Processes.Default.Streams.All = Testers.ContainsExpression('curl test', 'Making sure the basics still work')
ts1.Disk.traffic_out.Content = Testers.ContainsExpression('Generate initial STEK succeeded', 'should succeed')
ts2.Disk.traffic_out.Content = Testers.ContainsExpression('Generate initial STEK succeeded', 'should succeed')
ts3.Disk.traffic_out.Content = Testers.ContainsExpression('Generate initial STEK succeeded', 'should succeed')
ts4.Disk.traffic_out.Content = Testers.ContainsExpression('Generate initial STEK succeeded', 'should succeed')
ts5.Disk.traffic_out.Content = Testers.ContainsExpression('Generate initial STEK succeeded', 'should succeed')
tr1.StillRunningAfter = server
tr1.StillRunningAfter += ts1
tr1.StillRunningAfter += ts2
tr1.StillRunningAfter += ts3
tr1.StillRunningAfter += ts4
tr1.StillRunningAfter += ts5

tr2 = Test.AddTestRun("TLSv1.2 Session Ticket")
tr2.Command = \
    'echo -e "GET / HTTP/1.1\r\n" | openssl s_client -tls1_2 -connect 127.0.0.1:{0} -sess_out {5} && ' \
    'echo -e "GET / HTTP/1.1\r\n" | openssl s_client -tls1_2 -connect 127.0.0.1:{0} -sess_in {5} && ' \
    'echo -e "GET / HTTP/1.1\r\n" | openssl s_client -tls1_2 -connect 127.0.0.1:{1} -sess_in {5} && ' \
    'echo -e "GET / HTTP/1.1\r\n" | openssl s_client -tls1_2 -connect 127.0.0.1:{2} -sess_in {5} && ' \
    'echo -e "GET / HTTP/1.1\r\n" | openssl s_client -tls1_2 -connect 127.0.0.1:{3} -sess_in {5} && ' \
    'echo -e "GET / HTTP/1.1\r\n" | openssl s_client -tls1_2 -connect 127.0.0.1:{4} -sess_in {5}' \
    .format(
        ts1.Variables.ssl_port,
        ts2.Variables.ssl_port,
        ts3.Variables.ssl_port,
        ts4.Variables.ssl_port,
        ts5.Variables.ssl_port,
        os.path.join(Test.RunDirectory, 'sess.dat')
    )
tr2.ReturnCode = 0
tr2.Processes.Default.Streams.All.Content = Testers.Lambda(check_session)



'''
Verify ATS parent.config dest_ip dns bug fix.
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

Test.Summary = '''
Verify ATS parent_config with dest_ip=...
'''

# Create origin
origin = Test.MakeOriginServer("origin")

# default root
request_header_chk = {
    "headers": "GET / HTTP/1.1\r\n" + "Host: ats\r\n" + "\r\n",
    "timestamp": "1469733493.993",
    "body": "",
}

response_header_chk = {
    "headers": "HTTP/1.1 200 OK\r\n" + "Connection: close\r\n" + "\r\n",
    "timestamp": "1469733493.993",
    "body": "",
}

origin.addResponse("sessionlog.json", request_header_chk, response_header_chk)

request_header = {
    "headers":
        f"GET /foo.txt HTTP/1.1\r\n"
        "Host: does.not.matter\r\n"  # But cant be omitted
        "\r\n",
    "timestamp": "1469733493.993",
    "body": ""
}

response_header = {
    "headers": "HTTP/1.1 200 OK\r\n"
               "Connection: close\r\n"
               "Cache-control: max-age=60\r\n"
               "\r\n",
    "timestamp": "1469733493.993",
    "body": f"This is the body for foo.txt\n"
}
origin.addResponse("sessionlog.json", request_header, response_header)

# Configure DNS for cache layering
dns = Test.MakeDNServer("dns")
dns.addRecords(records={f"origin": ["127.0.0.1"]})
dns.addRecords(records={f"ts1": ["127.0.0.1"]})
dns.addRecords(records={f"ts0": ["127.0.0.1"]})
dns.addRecords(records={f"foo.bar": ["142.250.72.14"]})  # google.com

# Configure Traffic Server Mid
ts1 = Test.MakeATSProcess("ts1")
ts1.Disk.remap_config.AddLine(f"map / http://origin:{origin.Variables.Port}")

ts1.Disk.records_config.update(
    {
        'proxy.config.diags.debug.enabled': 1,
        'proxy.config.diags.debug.tags': 'http|dns|hostdb|parent',
        'proxy.config.dns.nameservers': f"127.0.0.1:{dns.Variables.Port}",
        'proxy.config.dns.resolv_conf': "NULL",
        'proxy.config.hostdb.lookup_timeout': 2,
        'proxy.config.http.connect_attempts_timeout': 1,
        'proxy.config.http.parent_proxy.self_detect': 0,
        'proxy.config.http.insert_response_via_str': 1,
        'proxy.config.proxy_name': 'ts1',
    })

# Configure Traffic Server Edge
ts0 = Test.MakeATSProcess("ts0")
ts0.Disk.remap_config.AddLine("map http://foo.bar http://foo.bar")

ts0.Disk.records_config.update(
    {
        'proxy.config.diags.debug.enabled': 1,
        'proxy.config.diags.debug.tags': 'http|dns|hostdb|parent',
        'proxy.config.dns.nameservers': f"127.0.0.1:{dns.Variables.Port}",
        'proxy.config.dns.resolv_conf': "NULL",
        'proxy.config.hostdb.lookup_timeout': 2,
        'proxy.config.http.connect_attempts_timeout': 1,
        'proxy.config.http.parent_proxy.self_detect': 0,
        'proxy.config.http.insert_response_via_str': 1,
        'proxy.config.proxy_name': 'ts0',
    })

ts0.Disk.parent_config.AddLines(
    [
        f"dest_ip=93.184.216.34 port=80 go_direct=true",  # example.com
        f'dest_host=foo.bar port=80 parent="ts1:{ts1.Variables.port}|1;" go_direct="false" parent_is_proxy="true"',
    ])

# Start everything up
tr = Test.AddTestRun("init")
tr.Processes.Default.StartBefore(origin)
tr.Processes.Default.StartBefore(dns)
tr.Processes.Default.StartBefore(ts0)
tr.Processes.Default.StartBefore(ts1)
tr.Processes.Default.Command = 'echo start TS, TSH_N, HTTP origin and DNS.'
tr.Processes.Default.ReturnCode = 0

curl_and_args = f"curl -s -D /dev/stdout -o /dev/stderr -x http://127.0.0.1:{ts0.Variables.port}"

# Request asset that goes through the layers
tr = Test.AddTestRun("request")
ps = tr.Processes.Default
ps.Command = curl_and_args + ' http://foo.bar/foo.txt'
ps.ReturnCode = 0
ps.Streams.stdout.Content = Testers.ContainsExpression("Via:.* ts1 .* ts0 ", "expected via header")
tr.StillRunningAfter = ts0



'''
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

Test.Summary = '''
Test next hop selection using strategies.yaml with consistent hashing.
'''

# Define and populate MicroServer.
#
server = Test.MakeOriginServer("server")
response_header = {
    "headers": "HTTP/1.1 200 OK\r\n"
               "Connection: close\r\n"
               "Cache-control: max-age=2\r\n"
               "\r\n",
    "timestamp": "1469733493.993",
    "body": "This is the body.\n"
}

request_header = {
    "headers":
        f"GET /obj0 HTTP/1.1\r\n"
        "Host: does.not.matter\r\n"  # But cannot be omitted.
        "\r\n",
    "timestamp": "1469733493.993",
    "body": ""
}
server.addResponse("sessionlog.json", request_header, response_header)

dns = Test.MakeDNServer("dns")

# Define next hop trafficserver instances.
#
ts_nh = Test.MakeATSProcess(f"ts_nh0", use_traffic_out=False, command=f"traffic_server 2>nh_trace.log")
ts_nh.Disk.records_config.update(
    {
        'proxy.config.diags.debug.enabled': 1,
        'proxy.config.diags.debug.tags': 'http|dns',
        'proxy.config.dns.nameservers': f"127.0.0.1:{dns.Variables.Port}",
        'proxy.config.dns.resolv_conf': "NULL",
    })
ts_nh.Disk.remap_config.AddLine(f"map / http://127.0.0.1:{server.Variables.Port}")
ts_nh.ReturnCode = Any(0, -2)

ts = Test.MakeATSProcess("ts")

ts.Disk.records_config.update(
    {
        'proxy.config.diags.debug.enabled': 1,
        'proxy.config.diags.debug.tags': 'http|dns|parent|next_hop|host_statuses|hostdb',
        'proxy.config.dns.nameservers': f"127.0.0.1:{dns.Variables.Port}",  # Only nameservers if resolv_conf NULL.
        'proxy.config.dns.resolv_conf': "NULL",  # This defaults to /etc/resvolv.conf (OS namesevers) if not NULL.
        'proxy.config.http.cache.http': 1,
        'proxy.config.http.uncacheable_requests_bypass_parent': 0,
        'proxy.config.http.no_dns_just_forward_to_parent': 1,
        'proxy.config.http.parent_proxy.mark_down_hostdb': 0,
        'proxy.config.http.parent_proxy.self_detect': 0,
    })

ts.Disk.File(ts.Variables.CONFIGDIR + "/strategies.yaml", id="strategies", typename="ats:config")
s = ts.Disk.strategies
s.AddLine("groups:")
s.AddLine("  - &g1")
dns.addRecords(records={"next_hop0": ["127.0.0.1"]})
s.AddLine(f"    - host: next_hop0")
s.AddLine(f"      protocol:")
s.AddLine(f"        - scheme: http")
s.AddLine(f"          port: {ts_nh.Variables.port}")
# The health check URL does not seem to be used currently.
# s.AddLine(f"          health_check_url: http://next_hop0:{ts_nh.Variables.port}")
s.AddLine(f"      weight: 1.0")
s.AddLines(
    [
        "strategies:",
        "  - strategy: the-strategy",
        "    policy: consistent_hash",
        "    hash_key: path",
        "    go_direct: false",
        "    parent_is_proxy: true",
        "    ignore_self_detect: true",
        "    groups:",
        "      - *g1",
        "    scheme: http",
    ])

# Fallover not currently tested.
#
# s.AddLines([
# "    fallover:",
# "      max_simple_retries: 2",
# "      ring_mode: exhaust_ring",
# "      response_codes:",
# "        - 404",
# "      health_check:",
# "        - passive",])

ts.Disk.remap_config.AddLine("map http://dummy.com http://not_used @strategy=the-strategy")

tr = Test.AddTestRun()
tr.Processes.Default.StartBefore(server)
tr.Processes.Default.StartBefore(dns)
tr.Processes.Default.StartBefore(ts_nh)
tr.Processes.Default.StartBefore(Test.Processes.ts)
tr.Processes.Default.Command = 'echo start TS, HTTP server, DNS server and next hop TSes'
tr.Processes.Default.ReturnCode = 0

tr = Test.AddTestRun()
tr.Processes.Default.Command = (f'curl --verbose --proxy 127.0.0.1:{ts.Variables.port} http://dummy.com/obj0')
tr.Processes.Default.Streams.stdout = "body.gold"
tr.Processes.Default.ReturnCode = 0

# Wait enough time that the item should be aged out of the cache.
tr = Test.AddTestRun("Wait for cached object to be stale.")
tr.Processes.Default.Command = "sleep 4"
tr.StillRunningAfter = ts

# Request should come back as 200
tr = Test.AddTestRun()
tr.Processes.Default.Command = (f'curl --verbose --proxy 127.0.0.1:{ts.Variables.port} http://dummy.com/obj0')
tr.Processes.Default.Streams.stdout = "body.gold"
tr.Processes.Default.ReturnCode = 0

tr = Test.AddTestRun()
tr.Processes.Default.Command = "grep 'Stale in cache' nh_trace*.log | cat"
tr.Processes.Default.Streams.stdout.Content = Testers.ContainsExpression("Stale in cache", "Expected stale content to be evaluated")
tr.Processes.Default.ReturnCode = 0



"""A gRPC client."""

#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import argparse
import asyncio
import grpc
import os
import sys

import simple_pb2
import simple_pb2_grpc

global_message_counter: int = 0
global_num_completed_connections: int = 0


async def run_grpc_client(hostname: str, proxy_port: int, proxy_cert: bytes) -> int:
    """Run the gRPC client.

    :param hostname: The hostname to which to connect.
    :param proxy_port: The ATS port to which to connect.
    :param proxy_cert: The public TLS certificate to verify ATS against.
    :return: The exit code.
    """
    global global_message_counter
    global global_num_completed_connections
    credentials = grpc.ssl_channel_credentials(root_certificates=proxy_cert)
    channel_options = (('grpc.ssl_target_name_override', hostname),)
    destination_endpoint = f'127.0.0.1:{proxy_port}'
    async with grpc.aio.secure_channel(destination_endpoint, credentials, options=channel_options) as channel:
        print(f'Connecting to: {destination_endpoint}')
        stub = simple_pb2_grpc.TalkerStub(channel)

        print(f'Creating two messages to send for counter: {global_message_counter}')
        message_1 = simple_pb2.SimpleRequest(message=f'Client request message: {global_message_counter}.1')
        message_2 = simple_pb2.SimpleRequest(message=f'Client request message: {global_message_counter}.2')
        my_message_count = global_message_counter
        global_message_counter += 1
        print(f'Sending request: {my_message_count}.1')
        response = await stub.MakeRequest(message_1)
        print(f'Response {my_message_count}.1 received from server: {response.message}')
        print(f'Sending the second request: {my_message_count}.2')
        message = simple_pb2.SimpleRequest(message=f'Client request message: {global_message_counter}.2')
        response = await stub.MakeAnotherRequest(message_2)
        print(f'Response {my_message_count}.2 received from server: {response.message}')
        global_num_completed_connections += 1
    return 0


async def run_grpc_clients(hostname: str, proxy_port: int, proxy_cert: bytes, num_connections: int) -> int:
    """Run the gRPC client.

    :param hostname: The hostname to which to connect.
    :param proxy_port: The ATS port to which to connect.
    :param proxy_cert: The public TLS certificate to verify ATS against.
    :param num_connections: The number of client connections to create.
    :return: The exit code.
    """
    tasks: list[asyncio.Task] = []
    for i in range(num_connections):
        print(f'Creating client {i}')
        tasks.append(run_grpc_client(hostname, proxy_port, proxy_cert))
    await asyncio.gather(*tasks)
    if global_num_completed_connections != num_connections:
        print(f'Expected {num_connections} responses, but got {global_num_completed_connections}')
        return 1
    else:
        print(f'Got the expected {num_connections} responses.')
        return 0


def parse_args() -> argparse.Namespace:
    """Parse command line arguments."""
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument('hostname', help='The hostname to which to connect.')

    parser.add_argument('proxy_port', metavar='proxy-port', type=int, help='The ATS port to which to connect.')
    parser.add_argument('proxy_cert', metavar='proxy-cert', type=argparse.FileType('rb'), help='The public TLS certificate to use.')
    parser.add_argument('num_connections', metavar='num-connections', type=int, help='The number of connections to create.')
    return parser.parse_args()


def main() -> int:
    """Run the main entry point for the gRPC client.

    :return: The exit code.
    """
    args = parse_args()

    try:
        return asyncio.run(run_grpc_clients(args.hostname, args.proxy_port, args.proxy_cert.read(), args.num_connections))
    except grpc.RpcError as e:
        print(f'RPC failed with code {e.code()}: {e.details()}')
        return 1


if __name__ == '__main__':
    sys.exit(main())



#!/usr/bin/env python3
"""Implements a client which slowly POSTs a request."""

#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

from http_utils import (wait_for_headers_complete, determine_outstanding_bytes_to_read, drain_socket)

import argparse
import socket
import sys


def parse_args() -> argparse.Namespace:
    """Parse the command line arguments."""
    parser = argparse.ArgumentParser()
    parser.add_argument("proxy_address", help="Address of the proxy to connect to.")
    parser.add_argument("proxy_port", type=int, help="The port of the proxy to connect to.")
    parser.add_argument(
        '-s',
        '--server-hostname',
        dest="server_hostname",
        default="some.server.com",
        help="The hostname of the server to connect to.")
    parser.add_argument("-t", "--send_time", dest="send_time", type=int, default=3, help="The number of seconds to send the POST.")
    parser.add_argument(
        '--finish-request',
        dest="finish_request",
        action='store_true',
        help="Finish sending the request before closing the connection.")

    return parser.parse_args()


def open_connection(address: str, port: int) -> socket.socket:
    """Open a connection to the desired host.

    :param address: The address of the host to connect to.
    :param port: The port of the host to connect to.
    :return: The socket connected to the host.
    """
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sock.connect((address, port))
    print(f'Connected to {address}:{port}')
    return sock


def send_slow_post(sock: socket.socket, server_hostname: str, send_time: int, finish_request: bool) -> None:
    """Send a slow POST request.

    :param sock: The socket to send the request on.
    :param server_hostname: The hostname of the server to connect to.
    :param send_time: The number of seconds to send the request.
    :param finish_request: Whether to finish sending the request before closing
    the connection.
    """
    # Send the POST request.
    host_header = f'Host: {server_hostname}\r\n'.encode()
    request = (b"POST / HTTP/1.1\r\n" + host_header + b"Transfer-Encoding: chunked\r\n"
               b"\r\n")
    sock.sendall(request)
    print('Sent request headers:')
    print(request.decode())

    print(f'Sending POST body for {send_time} seconds.')
    counter = 0
    while counter < send_time:
        # Send zero padded hex string of the counter.
        chunk = f'8\r\n{counter:08x}\r\n'.encode()
        sock.sendall(chunk)
        print(f'Sent chunk: {chunk.decode()}')
        counter += 1

    if finish_request:
        # Send the last chunk.
        sock.sendall(b'0\r\n\r\n')
    else:
        print('Aborting the request before sending the last chunk.')
        sock.close()


def drain_response(sock: socket.socket) -> None:
    """Drain the response from the server.

    :param sock: The socket to read the response from.
    """
    print('Waiting for the response to complete.')
    read_bytes = wait_for_headers_complete(sock)
    num_bytes_to_drain = determine_outstanding_bytes_to_read(read_bytes)
    drain_socket(sock, read_bytes, num_bytes_to_drain)
    print('Response complete.')


def main() -> int:
    """Run the client."""
    args = parse_args()
    print(args)

    with open_connection(args.proxy_address, args.proxy_port) as sock:
        send_slow_post(sock, args.server_hostname, args.send_time, args.finish_request)

        if args.finish_request:
            drain_response(sock)
    print('Done.')
    return 0


if __name__ == '__main__':
    sys.exit(main())



'''
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import os

Test.Summary = '''
Test for verify of runroot from traffic_layout.
'''

prefix = Test.Variables["PREFIX"]
bindir = Test.Variables["BINDIR"]
logdir = Test.Variables["LOGDIR"]

if bindir.startswith(prefix):
    # get the bin directory based on removing the common prefix
    binsuffix = bindir[len(prefix) + 1:]
else:
    # given a custom setup this might work.. or it might not
    binsuffix = bindir

if logdir.startswith(prefix):
    # get the bin directory based on removing the common prefix
    logsuffix = bindir[len(prefix) + 1:]
else:
    # given a custom setup this might work.. or it might not
    logsuffix = logdir

# create runroot
path = os.path.join(Test.RunDirectory, "runroot")
tr = Test.AddTestRun("Create runroot")
tr.Processes.Default.Command = "$ATS_BIN/traffic_layout init --path " + path
f = tr.Disk.File(os.path.join(path, "runroot.yaml"))
f.Exists = True

# verify test #1
tr = Test.AddTestRun("verify runroot test1")
tr.Processes.Default.Command = "$ATS_BIN/traffic_layout verify --path " + path

tr.Processes.Default.Streams.All = Testers.ContainsExpression(os.path.join(path, binsuffix), "example bindir output")
tr.Processes.Default.Streams.All = Testers.ContainsExpression(os.path.join(path, logsuffix), "example logdir output")
tr.Processes.Default.Streams.All = Testers.ContainsExpression("PASSED", "contain passed message")

# verify test #2
bin_path = Test.Variables.BINDIR[Test.Variables.BINDIR.find(Test.Variables.PREFIX) + len(Test.Variables.PREFIX) + 1:]
tr = Test.AddTestRun("verify runroot test2")
tr.Processes.Default.Command = "cd " + path + ";" + os.path.join(bin_path, "traffic_layout") + " verify --path " + path

tr.Processes.Default.Streams.All = Testers.ContainsExpression(os.path.join(path, binsuffix), "example bindir output")
tr.Processes.Default.Streams.All = Testers.ContainsExpression(os.path.join(path, logsuffix), "example logdir output")
tr.Processes.Default.Streams.All = Testers.ContainsExpression("PASSED", "contain passed message")



'''
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
import os
import yaml

Test.Summary = 'Testing ATS records to yaml script.'

file_suffix = 1

# case 1 TLS with no data
tr = Test.AddTestRun("Test records to yaml convert script - full file")

tr.Setup.Copy(os.path.join(Test.Variables.RepoDir, "tools/records/convert2yaml.py"))
tr.Setup.Copy('legacy_config/full_records.config')
tr.Processes.Default.Command = f'python3 convert2yaml.py -f full_records.config --output generated{file_suffix}.yaml --yaml --mute'
f = tr.Disk.File(f"generated{file_suffix}.yaml")
f.Content = "gold/full_records.yaml"

file_suffix = file_suffix + 1

tr = Test.AddTestRun("Test records to yaml convert script -only renamed records.")
tr.Setup.Copy(os.path.join(Test.Variables.RepoDir, "tools/records/convert2yaml.py"))
tr.Setup.Copy('legacy_config/old_records.config')
tr.Processes.Default.Command = f'python3 convert2yaml.py -f old_records.config --output generated{file_suffix}.yaml --yaml'
tr.Processes.Default.Stream = 'gold/renamed_records.gold'
f = tr.Disk.File(f"generated{file_suffix}.yaml")
f.Content = "gold/renamed_records.yaml"

tr = Test.AddTestRun("Test errors when trying to override values ")
tr.Setup.Copy(os.path.join(Test.Variables.RepoDir, "tools/records/convert2yaml.py"))
tr.Setup.Copy('legacy_config/override_value.config')
tr.Processes.Default.Command = f'python3 convert2yaml.py -f override_value.config --output generated{file_suffix}.yaml --yaml -m'
tr.Processes.Default.Streams.stdout += Testers.ContainsExpression(
    "We cannot continue with 'proxy.config.ssl.client.verify.server.policy' at line '3' as a value node will be overridden",
    "Error should be present")

tr = Test.AddTestRun("Test errors when trying to override maps")
tr.Setup.Copy(os.path.join(Test.Variables.RepoDir, "tools/records/convert2yaml.py"))
tr.Setup.Copy('legacy_config/override_map.config')
tr.Processes.Default.Command = f'python3 convert2yaml.py -f override_map.config --output generated{file_suffix}.yaml --yaml -m'
tr.Processes.Default.Streams.stdout += Testers.ContainsExpression(
    "We cannot continue with 'proxy.config.ssl.client.verify.server' at line '3' as an existing YAML map will be overridden.",
    "Error should be present")



# @file
#
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#
#  Copyright 2020, Verizon Media
#
import os.path

Test.Summary = '''
Basic remap testing.
'''

Test.SkipUnless(Condition.PluginExists("txn_box.so"))

tr = Test.TxnBoxTestAndRun(
    "Remap basics",
    "remap-base.replay.yaml",
    remap=[
        ['http://1.remap.ex/path', 'http://1.remapped.ex', ['--key=meta.txn_box.remap-1', 'remap-base.replay.yaml']],
        ['http://2.remap.ex/path', 'http://2.remapped.ex', ['--key=meta.txn_box.remap-2', 'remap-base.replay.yaml']],
        ['http://3.remap.ex/path', 'http://3.remapped.ex', ['--key=meta.txn_box.remap-3', 'remap-base.replay.yaml']],
        ['http://4.remap.ex/path', 'http://4.remapped.ex', ['--key=meta.txn_box.remap-4', 'remap-base.replay.yaml']],
        ['http://5.remap.ex/path', 'http://5.remapped.ex', ['--key=meta.txn_box.remap-5', 'remap-base.replay.yaml']],
        ['http://base.ex']
    ],
    enable_tls=True)

ts = tr.Variables.TS

ts.Setup.Copy("remap-base.replay.yaml", ts.Variables.CONFIGDIR)  # because it's remap only - not auto-copied.
ts.Setup.Copy("../ssl/server.key", os.path.join(ts.Variables.SSLDir, "server.key"))
ts.Setup.Copy("../ssl/server.pem", os.path.join(ts.Variables.SSLDir, "server.pem"))

ts = tr.Variables.TS
ts.Disk.records_config.update(
    {
        'proxy.config.diags.debug.enabled': 1,
        'proxy.config.diags.debug.tags': 'txn_box',
        'proxy.config.reverse_proxy.enabled': 1

        # enable ssl port
        ,
        'proxy.config.ssl.server.cert.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.ssl.server.private_key.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.http.server_ports': '{0} {1}:ssl'.format(ts.Variables.port, ts.Variables.ssl_port),
        'proxy.config.ssl.client.verify.server.policy': "disabled"
    })

ts.Disk.ssl_multicert_config.AddLine('dest_ip=* ssl_cert_name=server.pem ssl_key_name=server.key')



'''
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import os

Test.Summary = '''
Test tls
'''

# Define default ATS
ts = Test.MakeATSProcess("ts", enable_tls=True)
server = Test.MakeOriginServer("server")

# ssl-post is built via `make`. Here we copy the built binary down to the test
# directory so that the test runs in this file can use it.
Test.Setup.Copy(os.path.join(Test.Variables.AtsBuildGoldTestsDir, 'tls', 'ssl-post'))

requestLocation = "test2"
reHost = "www.example.com"

testName = ""

header_count = 378

header_string = "POST /post HTTP/1.1\r\nHost: www.example.com\r\nContent-Length:1000\r\n"

for i in range(0, header_count):
    header_string = "{1}header{0}:{0}\r\n".format(i, header_string)
header_string = "{0}\r\n".format(header_string)

post_body = ""
for i in range(0, 1000):
    post_body = "{0}0".format(post_body)

# Add info the origin server responses
server.addResponse(
    "sessionlog.json", {
        "headers": header_string,
        "timestamp": "1469733493.993",
        "body": post_body
    }, {
        "headers":
            "HTTP/1.1 200 OK\r\nServer: microserver\r\nConnection: close\r\nCache-Control: max-age=3600\r\nContent-Length: 2\r\n\r\n",
        "timestamp": "1469733493.993",
        "body": "ok"
    })

# add ssl materials like key, certificates for the server
ts.addDefaultSSLFiles()

ts.Disk.remap_config.AddLine('map / http://127.0.0.1:{0}'.format(server.Variables.Port))

ts.Disk.ssl_multicert_config.AddLine('dest_ip=* ssl_cert_name=server.pem ssl_key_name=server.key')
ts.Disk.records_config.update(
    {
        'proxy.config.ssl.server.cert.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.ssl.server.private_key.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.exec_thread.autoconfig.scale': 1.0,
        'proxy.config.diags.debug.enabled': 1,
        'proxy.config.diags.debug.tags': 'ssl',
    })

tr = Test.AddTestRun("Run-Test")
tr.Command = './ssl-post 127.0.0.1 40 {0} {1}'.format(header_count, ts.Variables.ssl_port)
tr.ReturnCode = 0
tr.Processes.Default.StartBefore(server)
tr.Processes.Default.StartBefore(Test.Processes.ts)
tr.Processes.Default.Streams.stdout = "gold/ssl-post.gold"
tr.StillRunningAfter = server



'''
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import os

Test.Summary = 'Test loading storage metrics'
Test.ContinueOnFail = True

# CAVEAT: Below test cases doesn't have multiple span cases that requires RAW devices.
test_cases = [
    {
        "case": 0,
        "description": "default config",
        "storage": '''
storage 256M
''',
        "volume": '''
# empty
'''
    }, {
        "case": 1,
        "description": "four equally devided volumes",
        "storage": '''
storage 1G
''',
        "volume":
            '''
volume=1 scheme=http size=25%
volume=2 scheme=http size=25%
volume=3 scheme=http size=25%
volume=4 scheme=http size=25%
'''
    }, {
        "case": 2,
        "description": "exclusive span",
        "storage": '''
storage 256M volume=1
''',
        "volume": '''
volume=1 scheme=http size=262144
''',
        "hosting": '''
hostname=* volume=1
'''
    }
]


class StorageMetricsTest:
    """
    Test loading storage.config and volume.config

    1. Spawn TS process with configs in test_cases
    2. Get 'proxy.process.cache.*' metrics
    3. Check with 'gold/storage_N_stdout.gold' file
    """

    def run(self):
        for config in test_cases:
            i = config["case"]
            ts = Test.MakeATSProcess(f"ts_{i}")
            ts.Disk.storage_config.AddLine(config["storage"])
            ts.Disk.volume_config.AddLine(config["volume"])
            if "hosting" in config:
                ts.Disk.hosting_config.AddLine(config["hosting"])
            ts.Disk.records_config.update({
                'proxy.config.diags.debug.enabled': 1,
                'proxy.config.diags.debug.tags': 'cache',
            })

            tr = Test.AddTestRun()

            tr.Processes.Default.StartBefore(ts)
            tr.Processes.Default.Env = ts.Env
            tr.Processes.Default.Command = 'traffic_ctl --debug metric match proxy.process.cache.'
            tr.Processes.Default.ReturnCode = 0
            tr.Processes.Default.Streams.stdout = f"gold/storage_metrics_{i}_stdout.gold"
            tr.StillRunningAfter = ts


StorageMetricsTest().run()



'''
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import os
import sys

Test.Summary = '''
Test a basic remap of a http/2 connection
'''

Test.SkipUnless(Condition.HasCurlFeature('http2'))
Test.ContinueOnFail = True

# ----
# Setup Origin Server
# ----
server = Test.MakeOriginServer("server")

# For Test Case 1 & 5 - /
server.addResponse(
    "sessionlog.json", {
        "headers": "GET / HTTP/1.1\r\nHost: www.example.com\r\n\r\n",
        "timestamp": "1469733493.993",
        "body": ""
    }, {
        "headers": "HTTP/1.1 200 OK\r\nServer: microserver\r\nConnection: close\r\n\r\n",
        "timestamp": "1469733493.993",
        "body": ""
    })

# For Test Case 2 - /bigfile
# Add info for the large H2 download test
server.addResponse(
    "sessionlog.json", {
        "headers": "GET /bigfile HTTP/1.1\r\nHost: www.example.com\r\n\r\n",
        "timestamp": "1469733493.993",
        "body": ""
    }, {
        "headers":
            "HTTP/1.1 200 OK\r\nServer: microserver\r\nConnection: close\r\nCache-Control: max-age=3600\r\nContent-Length: 191414\r\n\r\n",
        "timestamp": "1469733493.993",
        "body": ""
    })

# For Test Case 3 - /test2
server.addResponse(
    "sessionlog.json", {
        "headers": "GET /test2 HTTP/1.1\r\nHost: www.example.com\r\n\r\n",
        "timestamp": "1469733493.993",
        "body": ""
    }, {
        "headers": "HTTP/1.1 200 OK\r\nServer: microserver\r\nTransfer-Encoding: chunked\r\nConnection: close\r\n\r\n",
        "timestamp": "1469733493.993",
        "body": ""
    })

# For Test Case 6 - /postchunked
post_body = "12345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890"
server.addResponse(
    "sessionlog.json", {
        "headers": "POST /postchunked HTTP/1.1\r\nHost: www.example.com\r\n\r\n",
        "timestamp": "1469733493.993",
        "body": post_body
    }, {
        "headers": "HTTP/1.1 200 OK\r\nServer: microserver\r\nConnection: close\r\nContent-Length: 10\r\n\r\n",
        "timestamp": "1469733493.993",
        "body": "0123456789"
    })

# For Test Case 7 - /bigpostchunked
# Make a post body that will be split across at least two frames
big_post_body = "0123456789" * 131070
server.addResponse(
    "sessionlog.json", {
        "headers": "POST /bigpostchunked HTTP/1.1\r\nHost: www.example.com\r\n\r\n",
        "timestamp": "1469733493.993",
        "body": big_post_body
    }, {
        "headers": "HTTP/1.1 200 OK\r\nServer: microserver\r\nConnection: close\r\nContent-Length: 10\r\n\r\n",
        "timestamp": "1469733493.993",
        "body": "0123456789"
    })

big_post_body_file = open(os.path.join(Test.RunDirectory, "big_post_body"), "w")
big_post_body_file.write(big_post_body)
big_post_body_file.close()

# For Test Case 8 - /huge_resp_hdrs
server.addResponse(
    "sessionlog.json", {
        "headers": "GET /huge_resp_hdrs HTTP/1.1\r\nHost: www.example.com\r\n\r\n",
        "timestamp": "1469733493.993",
        "body": ""
    }, {
        "headers": "HTTP/1.1 200 OK\r\nServer: microserver\r\nConnection: close\r\nContent-Length: 6\r\n\r\n",
        "timestamp": "1469733493.993",
        "body": "200 OK"
    })

# For Test Case 9 - /status/204
server.addResponse(
    "sessionlog.json", {
        "headers": "GET /status/204 HTTP/1.1\r\nHost: www.example.com\r\n\r\n",
        "timestamp": "1469733493.993",
        "body": ""
    }, {
        "headers": "HTTP/1.1 204 No Content\r\nServer: microserver\r\nConnection: close\r\n\r\n",
        "timestamp": "1469733493.993",
        "body": ""
    })

# ----
# Setup ATS
# ----
ts = Test.MakeATSProcess("ts", enable_tls=True, enable_cache=False)

# add ssl materials like key, certificates for the server
ts.addDefaultSSLFiles()

ts.Setup.CopyAs('rules/huge_resp_hdrs.conf', Test.RunDirectory)
ts.Disk.remap_config.AddLine(
    'map /huge_resp_hdrs http://127.0.0.1:{0}/huge_resp_hdrs @plugin=header_rewrite.so @pparam={1}/huge_resp_hdrs.conf '.format(
        server.Variables.Port, Test.RunDirectory))

ts.Disk.remap_config.AddLine('map / http://127.0.0.1:{0}'.format(server.Variables.Port))

ts.Disk.ssl_multicert_config.AddLine('dest_ip=* ssl_cert_name=server.pem ssl_key_name=server.key')
ts.Disk.records_config.update(
    {
        'proxy.config.diags.debug.enabled': 1,
        'proxy.config.diags.debug.tags': 'http',
        'proxy.config.ssl.server.cert.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.ssl.server.private_key.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.http2.active_timeout_in': 3,
        'proxy.config.http2.max_concurrent_streams_in': 65535,
    })

ts.Setup.CopyAs('h2client.py', Test.RunDirectory)
ts.Setup.CopyAs('h2active_timeout.py', Test.RunDirectory)

# ----
# Test Cases
# ----

# Test Case 1:  basic H2 interaction
tr = Test.AddTestRun("basic H2 interaction")
tr.Processes.Default.Command = f'{sys.executable} h2client.py {ts.Variables.ssl_port} / --verify_default_body'
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.StartBefore(server)
tr.Processes.Default.StartBefore(Test.Processes.ts)
tr.Processes.Default.Streams.stdout = "gold/remap-200.gold"
tr.StillRunningAfter = server

# Test Case 2: Make sure all the big file gets back.  Regression test for issue 1646
tr = Test.AddTestRun("big file download")
tr.Processes.Default.Command = f'{sys.executable} h2client.py {ts.Variables.ssl_port} /bigfile --repeat 2 --verify_default_body'
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.stdout = "gold/bigfile.gold"
tr.StillRunningAfter = server

# Test Case 3: Chunked content
tr = Test.AddTestRun("chunked content")
tr.Processes.Default.Command = f'{sys.executable} h2client.py {ts.Variables.ssl_port} /test2 --print_body'
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.stdout = "gold/chunked.gold"
tr.StillRunningAfter = server

# NOTE: Skipping this test run because traffic-replay doesn't currently support H2
# Test Case 4: Multiple request
# client_path = os.path.join(Test.Variables.AtsTestToolsDir, 'traffic-replay/')
# tr = Test.AddTestRun("multiple request")
# tr.Processes.Default.Command = \
#     (f"{sys.executable} {client_path} -type h2 -log_dir {server.Variables.DataDir} "
#      f"-port {ts.Variables.port} -host '127.0.0.1' -s_port {ts.Variables.ssl_port} -v -colorize False")
# tr.Processes.Default.ReturnCode = 0
# tr.Processes.Default.Streams.stdout = "gold/replay.gold"
# tr.StillRunningAfter = server

# Test Case 5: h2_active_timeout
tr = Test.AddTestRun("h2_active_timeout")
tr.Processes.Default.Command = f'{sys.executable} h2active_timeout.py {ts.Variables.ssl_port} / 4'
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.All = "gold/active_timeout.gold"
tr.StillRunningAfter = server

# Test Case 6: Post with chunked body
# While HTTP/2 does not support Transfer-encoding we pass that into curl to encourage it to not set the content length
# on the post body
tr = Test.AddTestRun("post with chunked body")
tr.Processes.Default.Command = 'curl -s -k -H "Transfer-Encoding: chunked" -d "{0}" https://127.0.0.1:{1}/postchunked'.format(
    post_body, ts.Variables.ssl_port)
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.All = "gold/post_chunked.gold"
tr.StillRunningAfter = server

# Test Case 7: Post with big chunked body
# While HTTP/2 does not support Transfer-encoding we pass that into curl to encourage it to not set the content length
# on the post body
tr = Test.AddTestRun("post with big chunked body")
tr.Processes.Default.Command = 'curl -s -k -H "Transfer-Encoding: chunked" -d @big_post_body https://127.0.0.1:{0}/bigpostchunked'.format(
    ts.Variables.ssl_port)
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.All = "gold/post_chunked.gold"
tr.StillRunningAfter = server

# Test Case 8: Huge response header
tr = Test.AddTestRun("huge response header")
# Different versions of curl have "bytes data" at various places in the output.
# Normalize them by simply filtering out those lines since they are not
# important to this test.
tr.Processes.Default.Command = f'curl -vs -k --http2 https://127.0.0.1:{ts.Variables.ssl_port}/huge_resp_hdrs |& grep -v "bytes data"'
tr.Processes.Default.ReturnCode = 0
# Different versions of curl will have different cases for HTTP/2 field names.
tr.Processes.Default.Streams.stdout = Testers.GoldFile("gold/http2_8_stdout.gold", case_insensitive=True)
tr.StillRunningAfter = server

# Test Case 9: Header Only Response - e.g. 204
tr = Test.AddTestRun("header only response")
tr.Processes.Default.Command = 'curl -vs -k --http2 https://127.0.0.1:{0}/status/204'.format(ts.Variables.ssl_port)
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.stdout = "gold/http2_9_stdout.gold"
# Different versions of curl will have different cases for HTTP/2 field names.
tr.Processes.Default.Streams.stderr = Testers.GoldFile("gold/http2_9_stderr.gold", case_insensitive=True)
tr.StillRunningAfter = server



'''
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

Test.Summary = '''
Test interaction of H2 and chunked encoding
'''

Test.SkipUnless(
    Condition.HasProgram("nghttp", "Nghttp need to be installed on system for this test to work"),
    Condition.HasCurlFeature('http2'))
Test.ContinueOnFail = True

Test.GetTcpPort("upstream_port")
Test.GetTcpPort("upstream_port2")
Test.GetTcpPort("upstream_port3")

# Define default ATS
ts = Test.MakeATSProcess("ts", enable_tls=True)

# add ssl materials like key, certificates for the server
ts.addDefaultSSLFiles()

delay_server = Test.Processes.Process(
    "delay-server", "bash -c '" + Test.TestDirectory + "/delay-server.sh {} outserver1'".format(Test.Variables.upstream_port))
server2 = Test.Processes.Process(
    "server2", "bash -c '" + Test.TestDirectory + "/server2.sh {} outserver2'".format(Test.Variables.upstream_port2))
server3 = Test.Processes.Process(
    "server3", "bash -c '" + Test.TestDirectory + "/server3.sh {} outserver3'".format(Test.Variables.upstream_port3))

ts.Disk.records_config.update(
    {
        'proxy.config.diags.debug.enabled': 1,
        'proxy.config.diags.debug.tags': 'http',
        'proxy.config.ssl.server.cert.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.ssl.server.private_key.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.ssl.client.verify.server.policy': 'PERMISSIVE',
    })

ts.Disk.remap_config.AddLine('map /delay-chunked-response http://127.0.0.1:{0}'.format(Test.Variables.upstream_port))
ts.Disk.remap_config.AddLine('map /post-full http://127.0.0.1:{0}'.format(Test.Variables.upstream_port2))
ts.Disk.remap_config.AddLine('map /post-chunked http://127.0.0.1:{0}'.format(Test.Variables.upstream_port3))

ts.Disk.ssl_multicert_config.AddLine('dest_ip=* ssl_cert_name=server.pem ssl_key_name=server.key')

# Using netcat as a cheap origin server in case 1 so we can insert a delay in sending back the response.
# Replaced microserver for cases 2 and 3 as well because I was getting python exceptions when running
# microserver if chunked encoding headers were specified for the request headers

# H2 GET request
# chunked response without content-length
# delay before final chunk size
server1_out = Test.Disk.File("outserver1")
tr = Test.AddTestRun()
tr.Processes.Default.Command = 'nghttp -vv https://127.0.0.1:{}/delay-chunked-response'.format(ts.Variables.ssl_port)
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.StartBefore(Test.Processes.ts)
tr.Processes.Default.StartBefore(delay_server)
tr.Processes.Default.Streams.All = Testers.ExcludesExpression("RST_STREAM", "Delayed chunk close should not cause reset")
tr.Processes.Default.Streams.All += Testers.ExcludesExpression("content-length", "Should return chunked")
tr.Processes.Default.Streams.All += Testers.ContainsExpression(":status: 200", "Should get successful response")
tr.StillRunningAfter = ts
# No resets in the output
# No content lengths in the header

# HTTP2 POST: www.example.com Host, chunked body
server2_out = Test.Disk.File("outserver2")
tr = Test.AddTestRun()
tr.Processes.Default.StartBefore(server2)
tr.Processes.Default.Command = 'curl --http2 -k https://127.0.0.1:{}/post-full --verbose -H "Transfer-encoding: chunked" -d "Knock knock"'.format(
    ts.Variables.ssl_port)
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.All = Testers.ContainsExpression("HTTP/2 200", "Request should succeed")
tr.Processes.Default.Streams.All += Testers.ContainsExpression("content-length:", "Response should include content length")
server2_out = Testers.ContainsExpression("Transfer-Encoding: chunked", "Request should be chunked encoded")
# No content-length in header
# Transfer encoding to origin, but no content-length
# Content length on the way back

# HTTP2 POST: chunked post body and chunked response
server3_out = Test.Disk.File("outserver3")
tr = Test.AddTestRun()
tr.Processes.Default.StartBefore(server3)
tr.Processes.Default.Command = 'curl --http2 -k https://127.0.0.1:{}/post-chunked --verbose -H "Transfer-encoding: chunked" -d "Knock knock"'.format(
    ts.Variables.ssl_port)
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.Streams.All = Testers.ContainsExpression("HTTP/2 200", "Request should succeed")
tr.Processes.Default.Streams.All += Testers.ExcludesExpression("content-length:", "Response should not include content length")
server3_out = Testers.ContainsExpression("Transfer-Encoding: chunked", "Request should be chunked encoded")
# No content length in header
# Transfer encoding to origin, but no content-length
# No content length in the response



'''
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import os

# ----
# Setup Test
# ----
Test.Summary = '''
Test the Expect header in post
'''
# Require HTTP/2 enabled Curl
Test.SkipUnless(Condition.HasCurlFeature('http2'),)
Test.ContinueOnFail = True

# ----
# Setup httpbin Origin Server
# ----
replay_file = "replay/post-continue.replay.yaml"
server = Test.MakeVerifierServerProcess("server", replay_file)

# ----
# Setup ATS
# ----
ts = Test.MakeATSProcess("ts", enable_tls=True, enable_cache=False)

ts2 = Test.MakeATSProcess("ts2", enable_tls=True, enable_cache=False)

# add ssl materials like key, certificates for the server
ts.addDefaultSSLFiles()
ts2.addDefaultSSLFiles()

ts.Disk.remap_config.AddLine('map / http://127.0.0.1:{0}'.format(server.Variables.http_port))
ts.Disk.ssl_multicert_config.AddLine('dest_ip=* ssl_cert_name=server.pem ssl_key_name=server.key')
ts.Disk.records_config.update(
    {
        'proxy.config.ssl.server.cert.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.ssl.server.private_key.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.diags.debug.enabled': 1,
        'proxy.config.diags.debug.tags': 'http',
    })
ts2.Disk.remap_config.AddLine('map / http://127.0.0.1:{0}'.format(server.Variables.http_port))
ts2.Disk.ssl_multicert_config.AddLine('dest_ip=* ssl_cert_name=server.pem ssl_key_name=server.key')
ts2.Disk.records_config.update(
    {
        'proxy.config.ssl.server.cert.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.ssl.server.private_key.path': '{0}'.format(ts.Variables.SSLDir),
        'proxy.config.diags.debug.enabled': 0,
        'proxy.config.diags.debug.tags': 'http',
        'proxy.config.http.send_100_continue_response': 1
    })

big_post_body = "0123456789" * 131070
big_post_body_file = open(os.path.join(Test.RunDirectory, "big_post_body"), "w")
big_post_body_file.write(big_post_body)
big_post_body_file.close()

test_run = Test.AddTestRun("http1.1 POST small body with Expect header")
test_run.Processes.Default.StartBefore(server)
test_run.Processes.Default.StartBefore(ts)
test_run.Processes.Default.Command = 'curl -v -o /dev/null --http1.1 -H "uuid: post" -H "Expect: 100-continue" -d "small body" -k https://127.0.0.1:{0}/post'.format(
    ts.Variables.ssl_port)
test_run.Processes.Default.Streams.All = "gold/post-h1.gold"
test_run.Processes.Default.Streams.All += Testers.ContainsExpression("HTTP/1.1 100 continue", "Has Expect header")
test_run.Processes.Default.Streams.All += Testers.ContainsExpression("Expect: 100-continue", "Has Expect header")
test_run.StillRunningAfter = server
test_run.StillRunningAfter = ts
test_run.Processes.Default.ReturnCode = 0

test_run = Test.AddTestRun("http1.1 POST large body with Expect header")
test_run.Processes.Default.Command = 'curl -v -o /dev/null --http1.1 -H "uuid: post" -H "Expect: 100-continue" -d @big_post_body -k https://127.0.0.1:{0}/post'.format(
    ts.Variables.ssl_port)
test_run.Processes.Default.Streams.All = "gold/post-h1.gold"
test_run.Processes.Default.Streams.All += Testers.ContainsExpression("HTTP/1.1 100 continue", "Has Expect header")
test_run.Processes.Default.Streams.All += Testers.ContainsExpression("Expect: 100-continue", "Has Expect header")
test_run.StillRunningAfter = server
test_run.StillRunningAfter = ts
test_run.Processes.Default.ReturnCode = 0

test_run = Test.AddTestRun("http1.1 POST small body w/o Expect header")
test_run.Processes.Default.Command = 'curl -v -o /dev/null --http1.1 -H "uuid: post" -H "Expect:" -d "small body" -k https://127.0.0.1:{0}/post'.format(
    ts.Variables.ssl_port)
test_run.Processes.Default.Streams.All = "gold/post-h1.gold"
test_run.Processes.Default.Streams.All += Testers.ExcludesExpression("HTTP/1.1 100 continue", "Does not have Expect header")
test_run.Processes.Default.Streams.All += Testers.ExcludesExpression("Expect: 100-continue", "Does not have Expect header")
test_run.StillRunningAfter = server
test_run.StillRunningAfter = ts
test_run.Processes.Default.ReturnCode = 0

test_run = Test.AddTestRun("http1.1 POST large body w/o Expect header")
test_run.Processes.Default.Command = 'curl -v -o /dev/null --http1.1 -H "uuid: post" -H "Expect: " -d @big_post_body -k https://127.0.0.1:{0}/post'.format(
    ts.Variables.ssl_port)
test_run.Processes.Default.Streams.All = "gold/post-h1.gold"
test_run.Processes.Default.Streams.All += Testers.ExcludesExpression("HTTP/1.1 100 continue", "Does not have Expect header")
test_run.Processes.Default.Streams.All += Testers.ExcludesExpression("Expect: 100-continue", "Does not have Expect header")
test_run.StillRunningAfter = server
test_run.StillRunningAfter = ts
test_run.Processes.Default.ReturnCode = 0

test_run = Test.AddTestRun("http2 POST small body with Expect header")
test_run.Processes.Default.Command = 'curl -v -o /dev/null --http2 -H "uuid: post" -H "Expect: 100-continue" -d "small body" -k https://127.0.0.1:{0}/post'.format(
    ts.Variables.ssl_port)
test_run.Processes.Default.Streams.All = "gold/post-h2.gold"
test_run.Processes.Default.Streams.All += Testers.ContainsExpression("xpect: 100-continue", "Has expect header")
test_run.Processes.Default.Streams.All += Testers.ContainsExpression("HTTP/2 100", "Has Expect header")
test_run.StillRunningAfter = server
test_run.StillRunningAfter = ts
test_run.Processes.Default.ReturnCode = 0

test_run = Test.AddTestRun("http2 POST large body with Expect header")
test_run.Processes.Default.Command = 'curl -v -o /dev/null --http2 -H "uuid: post" -H "Expect: 100-continue" -d @big_post_body -k https://127.0.0.1:{0}/post'.format(
    ts.Variables.ssl_port)
test_run.Processes.Default.Streams.All = "gold/post-h2.gold"
test_run.Processes.Default.Streams.All += Testers.ContainsExpression("xpect: 100-continue", "Has expect header")
test_run.Processes.Default.Streams.All += Testers.ContainsExpression("HTTP/2 100", "Has Expect header")
test_run.StillRunningAfter = server
test_run.StillRunningAfter = ts
test_run.Processes.Default.ReturnCode = 0

test_run = Test.AddTestRun("http2 POST small body w/o Expect header")
test_run.Processes.Default.Command = 'curl -v -o /dev/null --http2 -H "uuid: post" -H "Expect: " -d "small body" -k https://127.0.0.1:{0}/post'.format(
    ts.Variables.ssl_port)
test_run.Processes.Default.Streams.All = "gold/post-h2.gold"
test_run.Processes.Default.Streams.All += Testers.ExcludesExpression("xpect: 100-continue", "Has expect header")
test_run.Processes.Default.Streams.All += Testers.ExcludesExpression("HTTP/2 100", "Has Expect header")
test_run.StillRunningAfter = server
test_run.StillRunningAfter = ts
test_run.Processes.Default.ReturnCode = 0

test_run = Test.AddTestRun("http2 POST large body w/o Expect header")
test_run.Processes.Default.Command = 'curl -v -o /dev/null --http2 -H "uuid: post" -H "Expect: " -d @big_post_body -k https://127.0.0.1:{0}/post'.format(
    ts.Variables.ssl_port)
test_run.Processes.Default.Streams.All = "gold/post-h2.gold"
test_run.Processes.Default.Streams.All += Testers.ExcludesExpression("xpect: 100-continue", "Has expect header")
test_run.Processes.Default.Streams.All += Testers.ExcludesExpression("HTTP/2 100", "Has Expect header")
test_run.StillRunningAfter = server
test_run.StillRunningAfter = ts
test_run.Processes.Default.ReturnCode = 0

# Do them all again against the TS that will return 100-continue immediately
test_run = Test.AddTestRun("http1.1 POST small body with Expect header, immediate")
test_run.Processes.Default.StartBefore(Test.Processes.ts2)
test_run.Processes.Default.Command = 'curl -v -o /dev/null --http1.1 -H "uuid: post" -H "Expect: 100-continue" -d "small body" -k https://127.0.0.1:{0}/post'.format(
    ts2.Variables.ssl_port)
test_run.Processes.Default.Streams.All = "gold/post-h1.gold"
test_run.Processes.Default.Streams.All += Testers.ContainsExpression("HTTP/1.1 100 Continue", "Has Expect header")
test_run.Processes.Default.Streams.All += Testers.ContainsExpression("Expect: 100-continue", "Has Expect header")
test_run.StillRunningAfter = server
test_run.StillRunningAfter = ts2
test_run.Processes.Default.ReturnCode = 0

test_run = Test.AddTestRun("http1.1 POST large body with Expect header, immediate")
test_run.Processes.Default.Command = 'curl -v -o /dev/null --http1.1 -H "uuid: post" -H "Expect: 100-continue" -d @big_post_body -k https://127.0.0.1:{0}/post'.format(
    ts2.Variables.ssl_port)
test_run.Processes.Default.Streams.All = "gold/post-h1.gold"
test_run.Processes.Default.Streams.All += Testers.ContainsExpression("HTTP/1.1 100 Continue", "Has Expect header")
test_run.Processes.Default.Streams.All += Testers.ContainsExpression("Expect: 100-continue", "Has Expect header")
test_run.StillRunningAfter = server
test_run.StillRunningAfter = ts2
test_run.Processes.Default.ReturnCode = 0

test_run = Test.AddTestRun("http1.1 POST small body w/o Expect header, immediate")
test_run.Processes.Default.Command = 'curl -v -o /dev/null --http1.1 -H "uuid: post" -H "Expect:" -d "small body" -k https://127.0.0.1:{0}/post'.format(
    ts2.Variables.ssl_port)
test_run.Processes.Default.Streams.All = "gold/post-h1.gold"
test_run.Processes.Default.Streams.All += Testers.ExcludesExpression("HTTP/1.1 100 Continue", "Has Expect header")
test_run.Processes.Default.Streams.All += Testers.ExcludesExpression("Expect 100-continue", "Has Expect header")
test_run.StillRunningAfter = server
test_run.StillRunningAfter = ts2
test_run.Processes.Default.ReturnCode = 0

test_run = Test.AddTestRun("http1.1 POST large body w/o Expect header, immediate")
test_run.Processes.Default.Command = 'curl -v -o /dev/null --http1.1 -H "uuid: post" -H "Expect: " -d @big_post_body -k https://127.0.0.1:{0}/post'.format(
    ts2.Variables.ssl_port)
test_run.Processes.Default.Streams.All = "gold/post-h1.gold"
test_run.Processes.Default.Streams.All += Testers.ExcludesExpression("HTTP/1.1 100 Continue", "Has Expect header")
test_run.Processes.Default.Streams.All += Testers.ExcludesExpression("Expect 100-continue", "Has Expect header")
test_run.StillRunningAfter = server
test_run.StillRunningAfter = ts2
test_run.Processes.Default.ReturnCode = 0

test_run = Test.AddTestRun("http2 POST small body with Expect header, immediate")
test_run.Processes.Default.Command = 'curl -v -o /dev/null --http2 -H "uuid: post" -H "Expect: 100-continue" -d "small body" -k https://127.0.0.1:{0}/post'.format(
    ts2.Variables.ssl_port)
test_run.Processes.Default.Streams.All = "gold/post-h2.gold"
test_run.Processes.Default.Streams.All += Testers.ContainsExpression("xpect: 100-continue", "Has expect header")
test_run.Processes.Default.Streams.All += Testers.ContainsExpression("HTTP/2 100", "Has Expect header")
test_run.StillRunningAfter = server
test_run.StillRunningAfter = ts2
test_run.Processes.Default.ReturnCode = 0

test_run = Test.AddTestRun("http2 POST large body with Expect header, immediate")
test_run.Processes.Default.Command = 'curl -v -o /dev/null --http2 -H "uuid: post" -H "Expect: 100-continue" -d @big_post_body -k https://127.0.0.1:{0}/post'.format(
    ts2.Variables.ssl_port)
test_run.Processes.Default.Streams.All = "gold/post-h2.gold"
test_run.Processes.Default.Streams.All += Testers.ContainsExpression("xpect: 100-continue", "Has expect header")
test_run.Processes.Default.Streams.All += Testers.ContainsExpression("HTTP/2 100", "Has Expect header")
test_run.StillRunningAfter = server
test_run.StillRunningAfter = ts2
test_run.Processes.Default.ReturnCode = 0

test_run = Test.AddTestRun("http2 POST small body w/o Expect header, immediate")
test_run.Processes.Default.Command = 'curl -v -o /dev/null --http2 -H "uuid: post" -H "Expect: " -d "small body" -k https://127.0.0.1:{0}/post'.format(
    ts2.Variables.ssl_port)
test_run.Processes.Default.Streams.All = "gold/post-h2.gold"
test_run.Processes.Default.Streams.All += Testers.ExcludesExpression("xpect: 100-continue", "Has expect header")
test_run.Processes.Default.Streams.All += Testers.ExcludesExpression("HTTP/2 100", "Has Expect header")
test_run.StillRunningAfter = server
test_run.StillRunningAfter = ts2
test_run.Processes.Default.ReturnCode = 0

test_run = Test.AddTestRun("http2 POST large body w/o Expect header, immediate")
test_run.Processes.Default.Command = 'curl -v -o /dev/null --http2 -H "uuid: post" -H "Expect: " -d @big_post_body -k https://127.0.0.1:{0}/post'.format(
    ts2.Variables.ssl_port)
test_run.Processes.Default.Streams.All = "gold/post-h2.gold"
test_run.Processes.Default.Streams.All += Testers.ExcludesExpression("xpect: 100-continue", "Has expect header")
test_run.Processes.Default.Streams.All += Testers.ExcludesExpression("HTTP/2 100", "Has Expect header")
test_run.StillRunningAfter = server
test_run.StillRunningAfter = ts2
test_run.Processes.Default.ReturnCode = 0



'''
Test DNS TTL behavior.
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import ports

Test.Summary = 'Test DNS TTL behavior'


class TtlDnsTest:
    single_transaction_replay = "replay/single_transaction.replay.yaml"
    server_error_replay = "replay/server_error.replay.yaml"
    process_counter = 1

    # The TTL to set for every resolved hostname.
    dnsTTL = 1

    # The DNS query timeout.
    queryTimeout = 1

    def __init__(self, configure_serve_stale=False, exceed_serve_stale=False):
        """
        Args:
            configure_serve_stale: (bool) Whether the ATS process should be configured to
            serve stale DNS entries.

            exceed_serve_stale: (bool) Configure the serve_stale timeout to be low
            enough that the timed out DNS response will not be used.
        """
        self.configure_serve_stale = configure_serve_stale
        self.exceed_serve_stale = exceed_serve_stale

        self.server_process_counter = TtlDnsTest.get_unique_process_counter()
        TtlDnsTest.process_counter += 1

        self.setupOriginServer()
        self.setupTS()

    @classmethod
    def get_unique_process_counter(cls):
        this_counter = cls.process_counter
        cls.process_counter += 1
        return this_counter

    def addDNSServerToTestRun(self, test_run):
        dns = test_run.MakeDNServer("dns", port=self.dns_port)
        dns.addRecords(records={'resolve.this.com': ['127.0.0.1']})
        return dns

    def setupOriginServer(self):
        self.server = Test.MakeVerifierServerProcess(f"server-{self.server_process_counter}", TtlDnsTest.single_transaction_replay)

    def setupTS(self):
        self.ts = Test.MakeATSProcess(f"ts-{self.server_process_counter}", enable_cache=False)
        self.dns_port = ports.get_port(self.ts, 'dns_port')
        self.ts.Disk.records_config.update(
            {
                "proxy.config.diags.debug.enabled": 1,
                "proxy.config.diags.debug.tags": "dns",
                'proxy.config.dns.nameservers': f'127.0.0.1:{self.dns_port}',
                'proxy.config.dns.resolv_conf': 'NULL',

                # Configure ATS to treat each resolved name to have a 1 second
                # time to live.
                "proxy.config.hostdb.ttl_mode": 1,
                "proxy.config.hostdb.timeout": self.dnsTTL,

                # MicroDNS will be down for the second transaction. Have ATS give
                # up trying to talk to it after one second.
                "proxy.config.hostdb.lookup_timeout": self.queryTimeout,
            })
        if self.configure_serve_stale:
            if self.exceed_serve_stale:
                stale_timeout = 1
            else:
                stale_timeout = 300

            self.ts.Disk.records_config.update({"proxy.config.hostdb.serve_stale_for": stale_timeout})
        self.ts.Disk.remap_config.AddLine(f"map / http://resolve.this.com:{self.server.Variables.http_port}/")

    def testRunWithDNS(self):
        tr = Test.AddTestRun()

        # Run the DNS server with this test run so it will not be running in
        # the next one.
        dns = self.addDNSServerToTestRun(tr)
        process_number = TtlDnsTest.get_unique_process_counter()
        tr.AddVerifierClientProcess(
            f"client-{process_number}", TtlDnsTest.single_transaction_replay, http_ports=[self.ts.Variables.port])

        tr.Processes.Default.StartBefore(dns)
        tr.Processes.Default.StartBefore(self.server)
        tr.Processes.Default.StartBefore(self.ts)

        tr.StillRunningAfter = dns
        tr.StillRunningAfter = self.server
        tr.StillRunningAfter = self.ts

    def testRunWithoutDNS(self):
        tr = Test.AddTestRun()

        # Delay running the second transaction for long enough to guarantee
        # that both the TTL and the DNS query timeout (lookup_timeout) are
        # exceeded.
        tr.DelayStart = 3

        # Will the stale resolved DNS response be used?
        if self.configure_serve_stale and not self.exceed_serve_stale:
            # Yes: expect a proxied transaction with a 200 OK response.
            replay_file = TtlDnsTest.single_transaction_replay
        else:
            # No: expect a 5xx response because the server name could not be
            # resolved.
            replay_file = TtlDnsTest.server_error_replay
        process_number = TtlDnsTest.get_unique_process_counter()
        tr.AddVerifierClientProcess(f"client-{process_number}", replay_file, http_ports=[self.ts.Variables.port])

        tr.StillRunningAfter = self.server
        tr.StillRunningAfter = self.ts

    def run(self):
        self.testRunWithDNS()
        self.testRunWithoutDNS()


TtlDnsTest().run()
TtlDnsTest(configure_serve_stale=True, exceed_serve_stale=False).run()
TtlDnsTest(configure_serve_stale=True, exceed_serve_stale=True).run()



'''
Verify support of external log rotation via SIGUSR2.
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import os
import sys

TS_PID_SCRIPT = 'ts_process_handler.py'


class Sigusr2Test:
    """
    Handle this test-specific Traffic Server configuration.
    """

    __ts_counter = 1
    __server = None

    def __init__(self):
        self.server = self.__configure_server()
        self.ts = self.__configure_traffic_server()

    def __configure_traffic_server(self):
        self._ts_name = "sigusr2_ts{}".format(Sigusr2Test.__ts_counter)
        Sigusr2Test.__ts_counter += 1
        self.ts = Test.MakeATSProcess(self._ts_name)
        self.ts.Disk.records_config.update(
            {
                'proxy.config.http.wait_for_cache': 1,
                'proxy.config.diags.debug.enabled': 1,
                'proxy.config.diags.debug.tags': 'log',
                'proxy.config.log.periodic_tasks_interval': 1,

                # All log rotation should be handled externally.
                'proxy.config.log.rolling_enabled': 0,
                'proxy.config.log.auto_delete_rolled_files': 0,
            })

        self.diags_log = self.ts.Disk.diags_log.AbsPath

        # Add content handles for the rotated logs.
        self.rotated_diags_log = self.diags_log + "_old"
        self.ts.Disk.File(self.rotated_diags_log, id="diags_log_old")

        self.log_dir = os.path.dirname(self.diags_log)

        self.ts.Disk.remap_config.AddLine(
            'map http://127.0.0.1:{0} http://127.0.0.1:{1}'.format(self.ts.Variables.port, self.server.Variables.Port))
        self.ts.Disk.logging_yaml.AddLine(
            '''
            logging:
              formats:
                - name: has_path
                  format: "%<pqu>: %<sssc>"
              logs:
                - filename: test_rotation
                  format: has_path
            ''')
        self.configured_log = os.path.join(self.log_dir, "test_rotation.log")
        self.ts.Disk.File(self.configured_log, id="configured_log")

        self.rotated_configured_log = self.configured_log + "_old"
        self.ts.Disk.File(self.rotated_configured_log, id="configured_log_old")
        self.ts.StartBefore(self.server)
        return self.ts

    def __configure_server(self):
        if Sigusr2Test.__server:
            return Sigusr2Test.__server
        server = Test.MakeOriginServer("server")
        Sigusr2Test.__server = server
        for path in ['/first', '/second', '/third']:
            request_header = {
                "headers": "GET {} HTTP/1.1\r\n"
                           "Host: does.not.matter\r\n\r\n".format(path),
                "timestamp": "1469733493.993",
                "body": ""
            }
            response_header = {
                "headers": "HTTP/1.1 200 OK\r\n"
                           "Connection: close\r\n"
                           "Cache-control: max-age=85000\r\n\r\n",
                "timestamp": "1469733493.993",
                "body": "xxx"
            }
            server.addResponse("sessionlog.json", request_header, response_header)
        return server

    def get_sigusr2_signal_command(self):
        """
        Return the command that will send a USR2 signal to the traffic server
        process.
        """
        return (f"{sys.executable} {TS_PID_SCRIPT} "
                f"--signal SIGUSR2 {self._ts_name}")


Test.Summary = '''
Verify support of external log rotation via SIGUSR2.
'''

Test.Setup.CopyAs(TS_PID_SCRIPT, Test.RunDirectory)

#
# Test 1: Verify SIGUSR2 behavior for system logs.
#
tr1 = Test.AddTestRun("Verify system logs can be rotated")

# Configure Server.
diags_test = Sigusr2Test()

# Configure our rotation processes.
rotate_diags_log = tr1.Processes.Process("rotate_diags_log", "mv {} {}".format(diags_test.diags_log, diags_test.rotated_diags_log))

# Configure the signaling of SIGUSR2 to traffic_server.
tr1.Processes.Default.Command = diags_test.get_sigusr2_signal_command()
tr1.Processes.Default.Return = 0
tr1.Processes.Default.Ready = When.FileExists(diags_test.diags_log)

# Configure process order.
tr1.Processes.Default.StartBefore(rotate_diags_log)
rotate_diags_log.StartBefore(diags_test.ts)
tr1.StillRunningAfter = diags_test.ts
tr1.StillRunningAfter = diags_test.server

# diags.log should have been rotated. The old one had the reference to traffic
# server running, this new one shouldn't. But it should indicate that the new
# diags.log was opened.
diags_test.ts.Disk.diags_log.Content += Testers.ExcludesExpression(
    "traffic server running", "The new diags.log should not reference the running traffic server")

diags_test.ts.Disk.diags_log.Content += Testers.ContainsExpression(
    "Reseated diags.log", "The new diags.log should indicate the newly opened diags.log")

#
# Test 2: Verify SIGUSR2 isn't needed for rotated configured logs.
#
tr2 = Test.AddTestRun("Verify yaml.log logs can be rotated")
configured_test = Sigusr2Test()

first_curl = tr2.Processes.Process(
    "first_curl", 'curl "http://127.0.0.1:{0}/first" --verbose'.format(configured_test.ts.Variables.port))
# Note that for each of these processes, aside from the final Default one, they
# are all treated like long-running servers to AuTest. Thus the long sleeps
# only allow us to wait until the logs get populated with the desired content,
# the test will not wait the entire time for them to complete.
first_curl_ready = tr2.Processes.Process("first_curl_ready", 'sleep 30')
# In the autest environment, it can take more than 10 seconds for the log file to be created.
first_curl_ready.StartupTimeout = 30
first_curl_ready.Ready = When.FileContains(configured_test.configured_log, "/first")

rotate_log = tr2.Processes.Process(
    "rotate_log_file", "mv {} {}".format(configured_test.configured_log, configured_test.rotated_configured_log))

second_curl = tr2.Processes.Process(
    "second_curl", 'curl "http://127.0.0.1:{0}/second" --verbose'.format(configured_test.ts.Variables.port))

second_curl_ready = tr2.Processes.Process("second_curl_ready", 'sleep 30')
# In the autest environment, it can take more than 10 seconds for the log file to be created.
second_curl_ready.StartupTimeout = 30
second_curl_ready.Ready = When.FileContains(configured_test.rotated_configured_log, "/second")

send_pkill = tr2.Processes.Process("Send_SIGUSR2", configured_test.get_sigusr2_signal_command())
send_pkill_ready = tr2.Processes.Process("send_pkill_ready", 'sleep 30')
send_pkill_ready.StartupTimeout = 30
send_pkill_ready.Ready = When.FileExists(configured_test.configured_log)

third_curl = tr2.Processes.Process(
    "third_curl", 'curl "http://127.0.0.1:{0}/third" --verbose'.format(configured_test.ts.Variables.port))
third_curl_ready = tr2.Processes.Process("third_curl_ready", 'sleep 30')
# In the autest environment, it can take more than 10 seconds for the log file to be created.
third_curl_ready.StartupTimeout = 30
third_curl_ready.Ready = When.FileContains(configured_test.configured_log, "/third")

tr2.Processes.Default.Command = "echo waiting for test processes to be done"
tr2.Processes.Default.Return = 0

# Configure process order:
#   1. curl /first. The entry should be logged to current log which will be _old.
#   2. mv the log to _old.
#   3. curl /second. The entry should end up in _old log.
#   4. Send a SIGUSR2 to traffic_server. The log should be recreated.
#   5. curl /third. The entry should end up in the new, non-old, log file.
#
tr2.Processes.Default.StartBefore(third_curl_ready)
third_curl_ready.StartBefore(third_curl)
third_curl.StartBefore(send_pkill_ready)
send_pkill_ready.StartBefore(send_pkill)
send_pkill.StartBefore(second_curl_ready)
second_curl_ready.StartBefore(second_curl)
second_curl.StartBefore(rotate_log)
rotate_log.StartBefore(first_curl_ready)
first_curl_ready.StartBefore(first_curl)
first_curl.StartBefore(configured_test.ts)
tr2.StillRunningAfter = configured_test.ts

# Verify that the logs are in the correct files.
configured_test.ts.Disk.configured_log.Content += Testers.ExcludesExpression(
    "/first", "The new test_rotation.log should not have the first GET retrieval in it.")
configured_test.ts.Disk.configured_log.Content += Testers.ExcludesExpression(
    "/second", "The new test_rotation.log should not have the second GET retrieval in it.")
configured_test.ts.Disk.configured_log.Content += Testers.ContainsExpression(
    "/third", "The new test_rotation.log should have the third GET retrieval in it.")

configured_test.ts.Disk.configured_log_old.Content += Testers.ContainsExpression(
    "/first", "test_rotation.log_old should have the first GET retrieval in it.")
configured_test.ts.Disk.configured_log_old.Content += Testers.ContainsExpression(
    "/second", "test_rotation.log_old should have the second GET retrieval in it.")
configured_test.ts.Disk.configured_log_old.Content += Testers.ExcludesExpression(
    "/third", "test_rotation.log_old should not have the third GET retrieval in it.")



'''
Test that redirects will be followed when refreshing stale cache objects.
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

Test.Summary = '''
Test that redirects will be followed when refreshing stale cache objects.
'''

server = Test.MakeOriginServer("server")

ArbitraryTimestamp = '12345678'

request_header = {
    'headers': ('GET /obj HTTP/1.1\r\n'
                'Host: *\r\n\r\n'),
    'timestamp': ArbitraryTimestamp,
    'body': ''
}
response_header = {
    'headers': ('HTTP/1.1 302 Found\r\n'
                'Location: http://127.0.0.1:{}/obj2\r\n\r\n'.format(server.Variables.Port)),
    'timestamp': ArbitraryTimestamp,
    'body': ''
}
server.addResponse('sessionfile.log', request_header, response_header)

request_header = {
    'headers': ('GET /obj2 HTTP/1.1\r\n'
                'Host: *\r\n\r\n'),
    'timestamp': ArbitraryTimestamp,
    'body': ''
}
response_header = {
    'headers': ('HTTP/1.1 200 OK\r\n'
                'X-Obj: obj2\r\n'
                'Cache-Control: max-age=2\r\n'
                'Content-Length: 0\r\n\r\n'),
    'timestamp': ArbitraryTimestamp,
    'body': ''
}
server.addResponse('sessionfile.log', request_header, response_header)

ts = Test.MakeATSProcess("ts")

ts.Disk.records_config.update(
    {
        'proxy.config.diags.debug.enabled': 1,
        'proxy.config.diags.debug.tags': 'http|dns|cache|redirect',
        'proxy.config.http.cache.required_headers': 0,  # Only Content-Length header required for caching.
        'proxy.config.http.push_method_enabled': 1,
        'proxy.config.url_remap.remap_required': 0,
        'proxy.config.http.redirect.actions': 'routable:follow,loopback:follow,self:follow',
        'proxy.config.http.number_of_redirections': 1
    })

# Set up to check the output after the tests have run.
#
log_id = Test.Disk.File("log2.txt")
log_id.Content = "gold/redirect_stale.gold"

tr = Test.AddTestRun()
tr.Processes.Default.StartBefore(ts)
tr.Processes.Default.StartBefore(server)
tr.Processes.Default.Command = (
    r"printf 'GET /obj HTTP/1.1\r\nHost: 127.0.0.1:{}\r\n\r\n' | nc localhost {} >> log.txt".format(
        server.Variables.Port, ts.Variables.port))
tr.Processes.Default.ReturnCode = 0

# Wait for the response in cache to become stale, then GET it again.
#
tr = Test.AddTestRun()
tr.Processes.Default.Command = (
    r"sleep 4 ; printf 'GET /obj HTTP/1.1\r\nHost: 127.0.0.1:{}\r\n\r\n' | nc localhost {} >> log.txt".format(
        server.Variables.Port, ts.Variables.port))
tr.Processes.Default.ReturnCode = 0

# Filter out inconsistent content in test output.
#
tr = Test.AddTestRun()
tr.Processes.Default.Command = (
    r"grep -v -e '^Date: ' -e '^Age: ' -e '^Connection: ' -e '^Server: ATS/' log.txt | tr -d '\r'> log2.txt")
tr.Processes.Default.ReturnCode = 0



'''
'''
#  Licensed to the Apache Software Foundation (ASF) under one
#  or more contributor license agreements.  See the NOTICE file
#  distributed with this work for additional information
#  regarding copyright ownership.  The ASF licenses this file
#  to you under the Apache License, Version 2.0 (the
#  "License"); you may not use this file except in compliance
#  with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import os
import time
from jsonrpc import Request

Test.Summary = '''
Test a basic regex_revalidate
'''

# Test description:
# Load up cache, ensure fresh
# Create regex reval rule, config reload:
#  ensure item is staled only once.
# Add a new rule, config reload:
#  ensure item isn't restaled again, but rule still in effect.
#
# If the rule disappears from regex_revalidate.conf its still loaded!!
# A rule's expiry can't be changed after the fact!

Test.SkipUnless(Condition.PluginExists('regex_revalidate.so'), Condition.PluginExists('xdebug.so'))
Test.ContinueOnFail = False

# configure origin server
server = Test.MakeOriginServer("server")

# Define ATS and configure
ts = Test.MakeATSProcess("ts")

Test.testName = "regex_revalidate"
Test.Setup.Copy("metrics.sh")

# default root
request_header_0 = {
    "headers": "GET / HTTP/1.1\r\n" + "Host: www.example.com\r\n" + "\r\n",
    "timestamp": "1469733493.993",
    "body": "",
}

response_header_0 = {
    "headers": "HTTP/1.1 200 OK\r\n" + "Connection: close\r\n" + "Cache-Control: max-age=300\r\n" + "\r\n",
    "timestamp": "1469733493.993",
    "body": "xxx",
}

# cache item path1
request_header_1 = {
    "headers": "GET /path1 HTTP/1.1\r\n" + "Host: www.example.com\r\n" + "\r\n",
    "timestamp": "1469733493.993",
    "body": ""
}
response_header_1 = {
    "headers":
        "HTTP/1.1 200 OK\r\n" + "Connection: close\r\n" + 'Etag: "path1"\r\n' + "Cache-Control: max-age=600,public\r\n" + "\r\n",
    "timestamp": "1469733493.993",
    "body": "abc"
}

# cache item path1a
request_header_2 = {
    "headers": "GET /path1a HTTP/1.1\r\n" + "Host: www.example.com\r\n" + "\r\n",
    "timestamp": "1469733493.993",
    "body": ""
}
response_header_2 = {
    "headers":
        "HTTP/1.1 200 OK\r\n" + "Connection: close\r\n" + 'Etag: "path1a"\r\n' + "Cache-Control: max-age=600,public\r\n" + "\r\n",
    "timestamp": "1469733493.993",
    "body": "cde"
}

# cache item path2a
request_header_3 = {
    "headers": "GET /path2a HTTP/1.1\r\n" + "Host: www.example.com\r\n" + "\r\n",
    "timestamp": "1469733493.993",
    "body": ""
}
response_header_3 = {
    "headers":
        "HTTP/1.1 200 OK\r\n" + "Connection: close\r\n" + 'Etag: "path2a"\r\n' + "Cache-Control: max-age=900,public\r\n" + "\r\n",
    "timestamp": "1469733493.993",
    "body": "efg"
}

server.addResponse("sessionlog.json", request_header_0, response_header_0)
server.addResponse("sessionlog.json", request_header_1, response_header_1)
server.addResponse("sessionlog.json", request_header_2, response_header_2)
server.addResponse("sessionlog.json", request_header_3, response_header_3)

# Configure ATS server
ts.Disk.plugin_config.AddLine('xdebug.so --enable=x-cache')
ts.Disk.plugin_config.AddLine('regex_revalidate.so -d -c regex_revalidate.conf')

regex_revalidate_conf_path = os.path.join(ts.Variables.CONFIGDIR, 'regex_revalidate.conf')
curl_and_args = 'curl -s -D - -v -H "x-debug: x-cache" -H "Host: www.example.com"'

path1_rule = 'path1 {}\n'.format(int(time.time()) + 600)

# Define first revision for when trafficserver starts
ts.Disk.File(
    regex_revalidate_conf_path, typename="ats:config").AddLines([
        "# Empty\n",
    ])

ts.Disk.remap_config.AddLine('map / http://127.0.0.1:{}'.format(server.Variables.Port))

# minimal configuration
ts.Disk.records_config.update(
    {
        'proxy.config.diags.debug.enabled': 1,
        'proxy.config.diags.debug.tags': 'regex_revalidate',
        #    'proxy.config.diags.debug.enabled': 0,
        'proxy.config.http.insert_age_in_response': 0,
        'proxy.config.http.response_via_str': 3,
    })

# 0 Test - Load cache (miss) (path1)
tr = Test.AddTestRun("Cache miss path1")
ps = tr.Processes.Default
ps.StartBefore(server, ready=When.PortOpen(server.Variables.Port))
ps.StartBefore(Test.Processes.ts)
ps.StartBefore(Test.Processes.ts)
ps.Command = curl_and_args + ' http://127.0.0.1:{}/path1'.format(ts.Variables.port)
ps.ReturnCode = 0
ps.Streams.stdout.Content = Testers.ContainsExpression("X-Cache: miss", "expected cache miss response")
tr.StillRunningAfter = ts

# 1 Test - Load cache (miss) for later test (path1a)
tr = Test.AddTestRun("Cache miss path1a")
ps = tr.Processes.Default
ps.Command = curl_and_args + ' http://127.0.0.1:{}/path1a'.format(ts.Variables.port)
ps.ReturnCode = 0
ps.Streams.stdout.Content = Testers.ContainsExpression("X-Cache: miss", "expected cache miss response")
tr.StillRunningAfter = ts

# 2 Test - Load cache (miss) for later test (path2a)
tr = Test.AddTestRun("Cache miss path2a")
ps = tr.Processes.Default
ps.Command = curl_and_args + ' http://127.0.0.1:{}/path2a'.format(ts.Variables.port)
ps.ReturnCode = 0
ps.Streams.stdout.Content = Testers.ContainsExpression("X-Cache: miss", "expected cache miss response")
tr.StillRunningAfter = ts

# 3 Test - Cache hit path1
tr = Test.AddTestRun("Cache hit fresh path1")
ps = tr.Processes.Default
ps.Command = curl_and_args + ' http://127.0.0.1:{}/path1'.format(ts.Variables.port)
ps.ReturnCode = 0
ps.Streams.stdout.Content = Testers.ContainsExpression("X-Cache: hit-fresh", "expected cache hit fresh response")
tr.StillRunningAfter = ts

# 4 Stage - Reload new regex_revalidate
tr = Test.AddTestRun("Reload config add path1")
# Need a sufficient delay so that the modification time difference of the new config file versus
# the old is greater than the granularity of the time stamp used.  (The config file write
# happens after the delay.)
tr.DelayStart = 1
tr.Disk.File(
    regex_revalidate_conf_path, typename="ats:config").AddLines([
        path1_rule,
    ])
tr.StillRunningAfter = ts
tr.StillRunningAfter = server
tr.AddJsonRPCClientRequest(ts, Request.admin_config_reload())
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.TimeOut = 5
tr.TimeOut = 5

# 5 Test - Revalidate path1
tr = Test.AddTestRun("Revalidate stale path1")
ps = tr.Processes.Default
tr.DelayStart = 5
ps.Command = curl_and_args + ' http://127.0.0.1:{}/path1'.format(ts.Variables.port)
ps.ReturnCode = 0
ps.Streams.stdout.Content = Testers.ContainsExpression("X-Cache: hit-stale", "expected cache hit stale response")
tr.StillRunningAfter = ts

# 6 Test - Cache hit (path1)
tr = Test.AddTestRun("Cache hit fresh path1")
ps = tr.Processes.Default
ps.Command = curl_and_args + ' http://127.0.0.1:{}/path1'.format(ts.Variables.port)
ps.ReturnCode = 0
ps.Streams.stdout.Content = Testers.ContainsExpression("X-Cache: hit-fresh", "expected cache hit fresh response")
tr.StillRunningAfter = ts

# 7 Stage - Reload new regex_revalidate
tr = Test.AddTestRun("Reload config add path2")
# Need a sufficient delay so that the modification time difference of the new config file versus
# the old is greater than the granularity of the time stamp used.  (The config file write
# happens after the delay.)
tr.DelayStart = 1
tr.Disk.File(
    regex_revalidate_conf_path, typename="ats:config").AddLines([
        path1_rule,
        'path2 {}\n'.format(int(time.time()) + 700),
    ])
tr.StillRunningAfter = ts
tr.StillRunningAfter = server
tr.AddJsonRPCClientRequest(ts, Request.admin_config_reload())
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.TimeOut = 5
tr.TimeOut = 5

# 8 Test - Cache hit (path1)
tr = Test.AddTestRun("Cache hit fresh path1")
tr.DelayStart = 5
ps = tr.Processes.Default
ps.Command = curl_and_args + ' http://127.0.0.1:{}/path1'.format(ts.Variables.port)
ps.ReturnCode = 0
ps.Streams.stdout.Content = Testers.ContainsExpression("X-Cache: hit-fresh", "expected cache hit fresh response")
tr.StillRunningAfter = ts

# 9 Test - Cache stale (check rule is still loaded) (path1a)
tr = Test.AddTestRun("Revalidate stale path1a")
ps = tr.Processes.Default
ps.Command = curl_and_args + ' http://127.0.0.1:{}/path1a'.format(ts.Variables.port)
ps.ReturnCode = 0
ps.Streams.stdout.Content = Testers.ContainsExpression("X-Cache: hit-stale", "expected cache hit stale response")
tr.StillRunningAfter = ts

# The C version of regex_revalidate doesn't allow an existing rule to
# be changed by a reload.

# 10 Stage - regex_revalidate rewrite rule early expire
tr = Test.AddTestRun("Reload config change path2")
# Need a sufficient delay so that the modification time difference of the new config file versus
# the old is greater than the granularity of the time stamp used.  (The config file write
# happens after the delay.)
tr.DelayStart = 1
tr.Disk.File(
    regex_revalidate_conf_path, typename="ats:config").AddLines([
        path1_rule,
        'path2 {}\n'.format(int(time.time()) - 100),
    ])
tr.StillRunningAfter = ts
tr.StillRunningAfter = server
tr.AddJsonRPCClientRequest(ts, Request.admin_config_reload())
tr.Processes.Default.ReturnCode = 0
tr.Processes.Default.TimeOut = 5
tr.TimeOut = 5

# 11 Test - Cache hit (path2a)
tr = Test.AddTestRun("Cache hit stale path2a")
tr.DelayStart = 5
ps = tr.Processes.Default
ps.Command = curl_and_args + ' http://127.0.0.1:{}/path2a'.format(ts.Variables.port)
ps.ReturnCode = 0
ps.Streams.stdout.Content = Testers.ContainsExpression("X-Cache: hit-stale", "expected cache hit stale response")
tr.StillRunningAfter = ts

# 12 Stats check
tr = Test.AddTestRun("Check stats")
tr.DelayStart = 5
ps = tr.Processes.Default
ps.Command = "bash -c ./metrics.sh"
ps.Env = ts.Env
ps.ReturnCode = 0
tr.StillRunningAfter = ts
